      subroutine aazvun
     & (lelli, ncl, stdns, htr, hc1, hc2, rcr, dbh, uuu, glmp, 
     & gg3, z0x, thx, aax)
c
c  Gap fraction at level z0x in direction thx, up.
c  A. Kuusk, 11.06.2021
c  Nilson, T., Kuusk, A., Lang, M., Pisek, J., and Kodar, A. (2011). 
c  Simulation of statistical characteristics of gap distribution in forest stands. 
c  Agricultural and Forest Meteorology, 151(7), 895-905.
c
c  Input: 
c    tree dimensions htrj, hc1j, hc2j, rcrj, dbhj, stand density stdnsj,
c    tlgj=gg3j*rlai(jcl)*clmpsh(jcl)+rbai(jcl), gg3 - G-function,
c    z0x - height, thx - zenith angle.
c  Output: 
c    aax - gap fraction at level z at zenith angle thx, up,
c    szx - tree projection at level z in direction thx
c    zrx - relative z-coordinate in a crown
c
      include 'frtpar.h'
      include 'nch.h'
c     implicit double precision (a-h, o-z)
      logical lelli, lellj
c
      dimension lelli(nclmax), stdns(nclmax), htr(nclmax),
     & hc1(nclmax), hc2(nclmax), rcr(nclmax), dbh(nclmax),
     & uuu(nclmax), glmp(nclmax)
      dimension gg3(nangls, nclmax)
c
      data eps/.1d-4/
c
      cthx  = cos(thx)
      tgthx = sin(thx)/cthx
      sumu  = 0.d0
c                                        crown projection
      do jcl = 1, ncl
        rllv2  = 0.d0
        lellj = lelli(jcl)
        htrj = htr(jcl)
        if (z0x .lt. htrj) then
          stdnsj = stdns(jcl)
          hc1j   = hc1(jcl)
          hc2j   = hc2(jcl)
          rcrj   = rcr(jcl)
          uuuj   = uuu(jcl)
          ulgv   = uuuj*gg3(1, jcl)
          dbhj   = dbh(jcl)
          glmpj  = glmp(jcl)
c
          call crproju
     &      (lellj, tgthx, z0x, htrj, hc1j, hc2j, rcrj, szx, vzx)
c
          rllv2 = vzx/szx/cthx
          azvu  = exp(-ulgv*rllv2)
          b11j = bij(azvu, glmpj)
c                                        stem projection
          if (tgthx .lt. eps) then
            styj = 0.d0
          else
            hbase = htrj - (hc1j + hc2j)
            call stem(z0x, hbase, dbhj, htrj, styx)
            styj = styx*tgthx
          endif
c
          sumu = sumu + stdnsj*(b11j*szx + styj)
        endif
      enddo
c
      aax = exp(-sumu)
c
      return
      end
*
************************************************************************
*
