      subroutine bck
     & (lelli, jcl, jths, ncl, shl, stdns, htr, dbh, hc1, hc2, rcr,
     & uuu, gg3, glmp, sthets, cthets, sthetv, cthetv, sphi,
     & cphi, calph, xi, yj, zk, sxuj, sxdj)
c
c     reflectance of the 1-st order scattering in tree crowns
c     1. ja"rku hajumise heleduskoefitsient (vo~rad)
c     Jo~eveer, (mai 88) Andrese progr.-st ilips xi, yj, zk
c
c    Bidirectional gap probability in a tree crown for the spherical
c    orientation of leaves. A. Kuusk, The hot spot effect in plant canopy
c    reflectance. - R. Myneny and J. Ross (Eds.), Photon-Vegetation
c    Interactions. Applications in Optical Remote sensing and Plant
c    Ecology. Springer, Berlin, 1991, 139-159.   A. Kuusk  4.01.1986
c    Modified 11.05.1998
c    Modified 12.2012 - elliptical orientation of foliage
c
c    lellips = .true. - ellipsoid, = .false. -- koonus + silinder
c    calph = cos(alph), alph - the angle between thets and thetv
c    shl - the hot spot parameter, the size of a leaf (shoot), m
c    sthet? = sin(thet?), cthet? = cos(thet?)
c    sphi = sin(phi), cphi = cos(phi), phi is the relative azimuth
c    aell - crown radius, helco - crown length, htrj - tree height, m
c
c    Input:
c     & (lelli, jcl, ncl, shl, stdns, htr, dbh, hc1, hc2, rcr,
c     & uuu, gg3, glmp, sthets, cthets, sthetv, cthetv, sphi,
c     & cphi, calph, xi, yj, zk,
c    Output:
c    sxuj, sxdj), sxuj - up, sxdj - down
c    zxr - local z-coordinate
c
      include 'frtpar.h'
c
      save chs1, chs3
c      
      logical lellj, lelli
      dimension lelli(nclmax)
c
      dimension shl(nclmax), stdns(nclmax), htr(nclmax), 
     & dbh(nclmax), hc1(nclmax), hc2(nclmax), rcr(nclmax), 
     & uuu(nclmax), glmp(nclmax)
      dimension gg3(nangls, nclmax)
c
      rintegr(xxx) = (1.d0 - exp(-xxx))/xxx
c
      lellj  = lelli(jcl)
      htrj   = htr(jcl)
      helco  = hc1(jcl)
      aell   = rcr(jcl)
      shlj   = shl(jcl)
c
c  sthedv = sthetv, cthedv = -cthetv, sphid = +sphi, 
c  cphid = -cphi, calphd = -calph
c
      if (lellj) then
         cell   = .5d0*helco
         zxr    = htrj - cell
c
         call rlips
     &    (xi, yj, zk, sthetv, cthetv, sphi, cphi, aell, cell, rlout)
         rllvu  = rlout
         call rlips
     &  (xi, yj, zk, sthetv, -cthetv, sphi, -cphi, aell, cell, rlout)
         rllvd  = rlout
c
         call rlips
     &    (xi, yj, zk, sthets, cthets, 0.d0, 1.d0, aell, cell, rlout)
         rlls = rlout
c
      else
         zxr   = htrj - helco
         vhcyl  = hc2(jcl)
c
         call rkoon
     &   (xi, yj, zk, sthetv, cthetv, sphi, cphi, 
     &    aell, helco, vhcyl, rlout)
         rllvu  = rlout
         call rkoon
     &   (xi, yj, zk, sthetv, -cthetv, sphi, -cphi, 
     &    aell, helco, vhcyl, rlout)
         rllvd  = rlout
c
         call rkoon
     &   (xi, yj, zk, sthets, cthets, 0.d0, 1.d0, 
     &    aell, helco, vhcyl, rlout)
         rlls = rlout
c
      endif
c
c                             ! hot-spot correction inside the crown
      uuuj  = uuu(jcl)
      ulgs  = uuuj*gg3(jths, jcl)
      ulgv  = uuuj*gg3(1, jcl)
      rl12u = (rllvu - rlls)**2 + (1.d0 - calph)*2.d0*rlls*rllvu
      if (rl12u .lt. 0.d0) then
         crr0 = 0.d0
      else
         crr0 = sqrt(rl12u)/shlj
      endif
      if (crr0 .lt. .001d0) then
         xtmp = 1.d0 - crr0*.5d0
      else
         xtmp = rintegr(crr0)
      endif
      chs1 = sqrt(rllvu*rlls)*xtmp
      xx  = (chs1 - rlls - rllvu)*sqrt(ulgs*ulgv)
      bgpu = exp(xx)
c
      rl12d = (rllvd - rlls)**2 + (1.d0 + calph)*2.d0*rlls*rllvd
      if (rl12d .lt. 0.d0) then
         crr0 = 0.d0
      else
         crr0 = sqrt(rl12d)/shlj
      endif
      if (crr0 .lt. .001d0) then
         xtmp = 1.d0 - crr0*.5d0
      else
         xtmp = rintegr(crr0)
      endif
      chs3 = sqrt(rllvd*rlls)*xtmp
      xx  = (chs3 - rlls - rllvd)*sqrt(ulgs*ulgv)
      bgpd = exp(xx)
c                bgp? - bidirectional gap probability inside the crown
c
c  Probabilities pooj  (A. Jõeveer)
c  the bididrectional gap probability outside the tree crown,
c  poouj - up, poodj - down
c
      x1  = xi + rllvu*sthetv*cphi
      y1  = yj + rllvu*sthetv*sphi
      z1  = zk + rllvu*cthetv + zxr
      x2  = xi + rlls*sthets
      y2  = yj
      z2  = zk + rlls*cthets + zxr
c
      x3  = xi - rllvd*sthetv*cphi
      y3  = yj + rllvd*sthetv*sphi
      z3  = zk - rllvd*cthetv + zxr
c
      call spooj
     & (lelli, ncl, jths, uuu, shl, stdns, htr, hc1, hc2, rcr, dbh,
     & gg3, glmp, x1, y1, z1, x2, y2, z2, x3, y3, z3,
     & rlls, rllvu, rllvd, sthets, cthets, sthetv, cthetv,
     & sphi, cphi, calph, chs1, chs3, poouj, poodj)
c
      sxuj  = bgpu*poouj
      sxdj  = bgpd*poodj
c
      return
      end
*
************************************************************************
*
