      subroutine  enel
     & (lelli, jcl, jths, ncl, shl, stdns, htr, dbh, hc1, hc2, rcr, 
     & uuu, gg3, glmp, thets, thetv, phi, nzs, 
     & bdgfu, bdgfd, btr1uk, btr1dk)
c
c   Single scattering in a tree crown,  A. Kuusk     29.08.2002
c   The bidirectional gap probability bgp*pooi is integrated 
c   over crown volume using Hammer-Stroud cubature in an ellipsoid,
c   Hammer, P.C., Stroud, A.H. Numerical evaluation of multiple
c   integrals II. Math. Tables and other Aids Comp. 12 (1958), No 64,
c   272-280,
c   or using Gauss-Legendre quadrature in z-direction and the Mysovskikh
c   (1981) algorithm 16.33 in x-y-plain in cone and cylinder.
c   The knots and weights for quadratures are in volint.h, 
c   the probabilities bgp and pooi are calculated in bck, 
c   the integrals are saved in the arrays bdgf?, and the probabilities to
c   see sunlit stem in the arrays btr1?k.
c
c   Elliptical orientation of foliage 12.2012.
c
c  Input:
c     & (lelli, jcl, jths, ncl, shl, stdns, htr, dbh, hc1, hc2, rcr,
c     & uuu, glmp, thets, thetv, phi, nzs
c  Output:
c    bdgfu, bdgfd, btr1uk, btr1dk
c
      include 'frtpar.h'
      include 'nch.h'
      include 'volint.h'
c
      logical lelli, lellj
      dimension lelli(nclmax)
c
      dimension shl(nclmax), stdns(nclmax), htr(nclmax), 
     & dbh(nclmax), hc1(nclmax), hc2(nclmax), rcr(nclmax), 
     & uuu(nclmax), glmp(nclmax)
      dimension bdgfu(nangls, nclmax), btr1uk(nangls, nclmax)
      dimension bdgfd(nangls, nclmax), btr1dk(nangls, nclmax)
      dimension gg3(nangls, nclmax)
c
      data xi, x2, x3, yj, y2, y3, rlls1, rllv1, rllv3, 
     &     chs1, chs3, cell /12*0.d0/
      data eps/0.1d-7/
*     data pi2/1.570796326794895d0/
c
      sthets = sin(thets)
      cthets = cos(thets)
      sthetv = sin(thetv)
      cthetv = cos(thetv)
      sphi   = sin(phi)
      cphi   = cos(phi)
      calph  = sthetv*sthets*cphi + cthetv*cthets
c         alph = acos(calph) is the angle betveen view directions 
c         thets and thetv
c
      lellj   = lelli(jcl)
      aell    = rcr(jcl)
      a2      = aell**2
      sumu    = 0.d0
      sumd    = 0.d0
      htrj    = htr(jcl)
      ntstz   = 2*nzs
c                                                       ellipsoid
      if (lellj) then
         hcrj   = hc1(jcl)
         cell   = hcrj*.5d0
         do itst = 1, netst
            xtsti = xetst(itst)*aell
            ytsti = yetst(itst)*aell
            ztsti = zetst(itst)*cell
            atsti = aetst(itst)*a2*cell
            call bck
     & (lelli, jcl, jths, ncl, shl, stdns, htr, dbh, hc1, hc2, rcr,
     & uuu, gg3, glmp, sthets, cthets, sthetv, cthetv, sphi,
     & cphi, calph, xtsti, ytsti, ztsti, sxuj, sxdj)
            sumu = sumu + sxuj*atsti
            sumd = sumd + sxdj*atsti
         enddo
         bdgfu(jths, jcl) = sumu
         bdgfd(jths, jcl) = sumd
      else
         hc1j   = hc1(jcl)
         vhcil  = hc2(jcl)
         hcrj   = hc1j + vhcil
         do itst = 1, ntstz
c                                                        cylinder
            if (vhcil .gt. 0d0) then
               sxyu   = 0.d0
               sxyd   = 0.d0
               aztsti = acztst(itst)/2.d0*vhcil
               ztsti  = (zctst(itst) - 1.d0)/2.d0*vhcil
               rcirc  = aell
               do jtst = 1, nctst
                  xtsti = xctst(jtst)*rcirc
                  ytsti = yctst(jtst)*rcirc
                  atsti = actst(jtst)*rcirc**2
                  call bck
     & (lelli, jcl, jths, ncl, shl, stdns, htr, dbh, hc1, hc2, rcr,
     & uuu, gg3, glmp, sthets, cthets, sthetv, cthetv, sphi,
     & cphi, calph, xtsti, ytsti, ztsti, sxuj, sxdj)
                  sxyu = sxyu + sxuj*atsti
                  sxyd = sxyd + sxdj*atsti
               enddo
               sumu = sumu + sxyu*aztsti
               sumd = sumd + sxyd*aztsti
            endif
c                                                       cone
            if (hc1j .gt. 0d0) then
               sxyu   = 0.d0
               sxyd   = 0.d0
               aztsti = acztst(itst)/2.d0*hc1j
               ztsti  = (zctst(itst) + 1.d0)/2.d0*hc1j
               rcirc  = aell*(1.d0 - ztsti/hc1j)
               do jtst = 1, nctst
                  xtsti = xctst(jtst)*rcirc
                  ytsti = yctst(jtst)*rcirc
                  atsti = actst(jtst)*rcirc**2
                  call bck
     & (lelli, jcl, jths, ncl, shl, stdns, htr, dbh, hc1, hc2, rcr,
     & uuu, gg3, glmp, sthets, cthets, sthetv, cthetv, sphi,
     & cphi, calph, xtsti, ytsti, ztsti, sxuj, sxdj)
                  sxyu = sxyu + sxuj*atsti
                  sxyd = sxyd + sxdj*atsti
               enddo
               sumu = sumu + sxyu*aztsti
               sumd = sumd + sxyd*aztsti
            endif
         enddo
         bdgfu(jths, jcl) = sumu
         bdgfd(jths, jcl) = sumd
      endif
c
c
c    The probability to see sunlit stem
c
      btr1u = 0.d0
      btr1d = 0.d0
      utyu  = 0.d0
      utyd  = 0.d0
c                       zvari - the height of the crown shade 
      tgths = sthets/cthets
      tgthx = max(tgths, eps)
      if (lellj) then
         zv = sqrt(a2 + (cell*tgths)**2)/tgthx
      else
         zv = aell/tgthx
      endif
      zvari = htrj - hcrj - zv
c
      if (zvari .gt. 0.d0) then
         if (thetv .lt. thets) then
            tgthv = sthetv/cthetv
            tgthx = max(tgthv, eps)
            if (lellj) then 
               xxx = sqrt(a2 + (cell*tgthv)**2)
               zv = xxx/tgthx
            else
               zv = aell/tgthx
            endif
            zview = max(zvari , 0.d0)
         else 
            zview = zvari
         endif
c
c                        ! uty - the area of sunlit stem
         utyd = dbh(jcl)*pi/2.d0*zvari
         utyu = dbh(jcl)*pi/2.d0*zview
c                               ! the integral of pooi over [0, zvari]
         dzenel = hcrj/ntstz
         if (zvari .le. dzenel) then
            dztrunk = zvari
         else
            dztrunk = dzenel
         endif
         nz = int(zvari/dztrunk + .5d0)
         nview = int(zview/dztrunk + .5d0) + 1
         dztrunk = zvari/nz
         xi      = 0.d0
         x2      = xi
         x3      = xi
         yj      = 0.d0
         y2      = yj
         y3      = yj
         rlls1   = 0.d0
         rllv1   = 0.d0
         rllv3   = 0.d0
         chs1    = 0.d0
         chs3    = 0.d0
c
         if (nview .lt. nz) then
            do 40 k = 1, nz + 1
               zk   = (k - 1)*dztrunk
               z2   = zk
               z3   = zk
c
      call spooj
     & (lelli, ncl, jths, uuu, shl, stdns, htr, hc1, hc2, rcr, dbh,
     & gg3, glmp, xi, yj, zk, x2, y2, z2, x3, y3, z3,
     & rlls1, rllv1, rllv3, sthets, cthets, sthetv, cthetv,
     & sphi, cphi, calph, chs1, chs3, poouj, poodj)
c
               if (k .eq. 1 .or. k .eq. nz) poodj = poodj/2.d0
               btr1d = btr1d + poodj
               if (k .eq. 1 .or. k .eq. nview) poouj = poouj/2.d0
               if (k .le. nview) btr1u = btr1u + poouj
40          continue
            btr1d = btr1d/nz
            btr1u = btr1u/nview
         else
            do 41 k = 1, nz + 1
               zk   = (k - 1)*dztrunk
               z2   = zk
               z3   = zk
c     
      call spooj
     & (lelli, ncl, jths, uuu, shl, stdns, htr, hc1, hc2, rcr, dbh,
     & gg3, glmp, xi, yj, zk, x2, y2, z2, x3, y3, z3,
     & rlls1, rllv1, rllv3, sthets, cthets, sthetv, cthetv,
     & sphi, cphi, calph, chs1, chs3, poouj, poodj)
c           
               if (k .eq. 1 .or. k .eq. nz) then
                  poodj = poodj/2.d0
                  poouj = poouj/2.d0
               endif
               btr1d = btr1d + poodj
               btr1u = btr1u + poouj
41          continue
            btr1d = btr1d/nz
            btr1u = btr1u/nz
         endif
      endif
      btr1uk(jths, jcl) = btr1u*utyu
      btr1dk(jths, jcl) = btr1d*utyd
c
      return
      end
*
************************************************************************
*
