      program frt13
c   Forest reflectance and transmittance, and its inversion.
c   (C) Andres Kuusk & Tiit Nilson
c   Kuusk, A. and Nilson, T. A directional multispectral forest 
c   reflectance model, Remote Sens. Environ., 2000, 72(2): 
c   244-252.
c   Version 04.2013.
c   Contact: A. Kuusk, Tartu Observatory, Estonia, andres@aai.ee
c
c   Aprill 2013: Lisasin inversiooni valiku ijob=6 - indikatrissi
c   inversioon ühel lainepikkusel ja ühel päikese seniitnurgal.
c
c   Januar 2023 added the warning: 
c   *** Warning!: canopy closure (CC) > crown closure (CR)
c     If canopy closure is greater than crown closure 
c     then the regularity parameter "tree  distr.  param    13" 
c     for this class is too large. 
c
c   Jaanuar 2013: Männil ja kuusel kasutatakse Henyey-Greensteini
c   faasifunktsiooni. Asümmeetria parameeter g (programmis ghg) on 
c   fikseeritud põhiprogrammi alguses.
c     ghg = 0.5d0  ! RAMI spruce, f = 0.829
c     ghg = 0.4d0  ! RAMI pine, f = 0.775, f - asümmeetriakoefitsient
c
cc   Jaanuar 2013: kui clmpsh(jcl)<0.9, siis võetakse elliptilise
cc   orientatsiooni faasifunktsiooni asemel M. Mõttuse (AFM 2012)
cc   okaspuu kasvu faasifunktsioon P_IL(Omega).  -- selle vist jätsin ära.
c
c   November-detsember 2012: Viisin sisse lehtede/okaste kaldenurkade 
c   jaotuse võrades - elliptiline orientatsioon G ja gamma 
c   analüütiliste lähenditega. Alustaimestiku mudeliks ACRMf. 
c   Muutus sisendfaili struktuur.
c
c   Juuli 2009: Abdelaziz Kallel leidis vea mulla arvestamises 
c   difuussetes voogudes. Parandasin dif3.f-is.
c   Lisasin moodulis layer.f lähendi väikese neeldumise jaoks.
c
c   Mai 2006: Mõned muudatused difuussete voogude arvutamises
c   mooduleis dif3() ja diffor(). frt.f read-direktiivides muutsin
c   formaadikirjeldusi, et töötaks ühtviisi unixeis ja DOS-is.
c
c   Aprill 2005: Viisin sisse puu tüve kuju arvestamise.
c   Puude paiknemise regulaarsust ei arvesta tüvede paiknemisel,
c   need paiknevad juhuslikult. Parandatatud paar initsialiseermiata
c   muutujat. Nüüd kompileeritav ka gfortran (Fortran95) 
c   kompilaatoriga. 
c
c   Jaanuar 2003: Vahetasin kubatuurvalemi ellipsoidi jaoks
c   cubell(5) -> cubell9
c
c   September 2002: Vahetasin difuussete voogude arvutuse 
c   Verhoefi mudeliga, parandasin vigu võrastiku alla suunatud 
c   heledustes.
c
c   August 2002: 3D-integraali arvutus kubatuurvalemitega.
c
c   Veebruar 2002: Lisatud leheoptika mudeli valik nii puudel 
c   kui aluspinnal. Alustaimestiku jaoks 2-kihiline mudel.
c   Lehekomponentide arv kasvatatud 10-ni.
c
c   Detsember 2001: Muutsin veateateid, paar väikest veaparandust.
c
c   Mai 2001: vahetasin PROSPECTI 1997 versiooni vastu.
c
c   Aprill 2001: Muutsin sihtfunktsiooni, lisandus eksperthinnang.
c
c   Detsember 2000: Tegin kolmest programmist (ifrt, frtspec, frtad)
c   ühe, muutus algandmete struktuur (slw, komponentide sisaldus
c   protsenti slw-st), järjestasin algandmete vektori teisiti.
c
c   September 1999: Hot-spoti parameeter antakse igale klassile
c   eraldi. Muutus algandmete fail.
c
c   Oktoober 1998: difuusse valgustatuse helduskoefitsient
c   arvutatakse phi = 90 deg korral polaarnurga järgi 
c   integreerides.
c
c   Juuli 1998: lisasin ühekordse hajumise tüvedelt. 
c   Muutsin pöördülesande vabade parameetrite arvu.
c
c   Mai 1998: Mudel töötab kõigi vaatesuundade jaoks. Sama programm
c   nii otsese kui pöördülesande jaoks. Väike muudatus
c   tüvede arvestamises pooi arvutamisel.
c
c   Märts 1998: Metsa läbipaistvus binoomvalemiga,
c   mitmekordse hajumise valemis efektiivne LAI Cheni 
c   valemitega, alustaimestiku heleduskoefitsient 
c   homogeense taimkatte mudeliga MCRM.
c
c   Märts, 1996: alustaimestiku peegeldumisspekter failis,
c   Price'i vektoreid otseselt ei kasuta, seega ilmutatud
c   parameetrite arv kahanes 14-le.
c
c   A. Kuusk 23.10.1995 Avignonis
c
c   August, 1995 Grignonis: algandmetes ka Q/D suhe, 
c   Pöördülesandes saab kasutada nii heleduskoefitsiendi
c   kui läbilaskvuskoefitsientide väärtusi. Vajaduse korral
c   tuleb muuta sihtfunktsiooni arvutamist funcifrt.F-is
c
      include 'frtpar.h'
      include 'nch.h'
      include 'func.h'
      include 'volint.h'
c
      logical lelli, lig, lml
      character outx*77, fname*77, xfs*72
      character*2 trl
      character*6 mv
      character*8 liik, elli, koon, kuju
      character*24 brreflf, trreflf
      character*72 clfile, oufile, cdumm
      character*40 frefr, fprice, fcoef1, fcoef2, fcoef3
      character*10 lmod1, lmod2, lmod3
      character*40 dname
      integer age
c
      dimension fcoef1(ncomp0), fcoef2(ncomp0), fcoef3(nclmax, ncomp0)
      dimension ccomp1(ncomp0), ccomp2(ncomp0)
      dimension lelli(nclmax), liik(nclmax), kuju(nclmax)
      dimension clfile(nclmax), lmod3(nclmax), ncomp3(nclmax),
     & clmpsh(nclmax)
c
      dimension brreflf(nclmax), trreflf(nclmax)
      dimension brrarr(nclmax, nspchnl), trrarr(nclmax, nspchnl)
c
      dimension dos(nsunmax)
      dimension wlarr(nspchnl)
      dimension thvr(nspchnl), phir(nspchnl)
c
      dimension bqjarr(nangls, nspchnl), drhoarr(nangls, nspchnl)
c
      dimension thknot(nangls), p14(nknots)
c
      dimension llxx(nvar), xmin(nvar), xmax(nvar), xx(nvar), 
     &          xa(nvar), xb(nvar), xxf(nvar), xdelt(nvar)
      dimension xc(nclmax, nvar)
c
      dimension sqarr(nangls, nspchnl)
c
      dimension sbgr(nangls), sq(nangls)
c
      character*20 tex
      character*30 tjob
      dimension tex(40), dcaer(4), tjob(7)
      dimension parr(nvar+1, nvar)
c
      dimension rthet(321), rsthet(321), bdown(321)
      dimension aavth(321), rcr1(321), rgr1(321), rdif(321)
c
      common /par/ zeps, tol, tiny, alphopt, betopt, gammopt, 
     &      ier, ncount, nfmax, itmax, itbr, nbrak, ilo
c
      data trl/'# '/, lig/.false./
      data elli/'  ellips'/, koon/'  cone+c'/
      data eps/.1d-2/, eps2/1.745d-2/, dthetv/.1d0/, thtv0/0.d0/
c
      data tex/' stand density, m-2', ' tree height, m', 
     & ' ellipsoid or cone', ' cylinder, m', ' crown radius, m',
     & ' trunk diameter, cm', ' total leaf weight',
     & ' leaf weight, g m-2', ' eln', ' thm', ' shoot size, m', 
     & ' BAI/LAI', ' tree distr. param.', ' g_H-G', 
     & ' c1, % of SLW', ' c2, % of SLW', ' c3, % of SLW',
     & ' c4, % of SLW', ' c5, % of SLW', ' c6, % of SLW',
     & ' c7, % of SLW', ' c8, % of SLW', ' c9, % of SLW',
     & ' c10, % of SLW', ' D_cell, µm', ' i-cell air', 
     & ' leaf str.par', ' refr. ind. ratio', 
     & ' ground LAI', ' leaf size', ' clmp', ' szz', ' eln', ' thm', 
     & ' n_ratio', ' SLW', ' s1_soil', ' s2', ' s3', ' s4'/
      data tjob /' - single', ' - spectrum', ' - angular distribution',
     & ' - several Sun angles', ' - inversion, relative errors', 
     & ' - inversion, absolute errors', ' - inversion, BRF'/
c
*     call fptrap(1)
c
      pi = acos(-1.d0)
      dr = pi/180.d0
c
      iwr    = 1
      ire    = 2
      ncount = 0
      nrefl  = 0
      ier    = 0
      iter   = 1
      js     = 1
      jp     = 1
      bangs  = -1.d0
      pkhair = .1d0
      dcell0 = 80.d0
      nzs    = 4
      nsangl = nknots + 1
      jwcl   = -1
c
c   Initialisation: the knots and weigths of Gauss-Legendre quadrature
      ntstz   = nknots - 1
      x1      = 0.d0
      x2      = pi/2.d0
      call gauleg(x1, x2, zctst, acztst, ntstz)
      thknot(1) = 0.d0
      p14(1)    = 0.d0
      do jth = 2, nknots
         thknot(jth) = zctst(jth - 1)
         p14(jth)    = cos(thknot(jth))*acztst(jth - 1)
      enddo
c
      call cubell9(netst, xetst, yetst, zetst, aetst)
      call cubcirc(nctst, xctst, yctst, actst)
c
      do jcl = 1, nclmax
         do jc = 1, ncomp0
            fcoef3(jcl, jc) = '_'
            xc(jcl, jc)     = 0.d0
            xmin(jc)        = 0.d0
            xmax(jc)        = 0.d0
            xdelt(jc)       = 0.d0
         enddo
         ncomp3(jcl) = 0
      enddo
c
      call getarg(1, clfile(1))
      call getarg(2, oufile)
      if (clfile(1) .eq. ' ') then
         write (0, '(a)')  ' Name of input file '
         read (*, '(a)')     clfile(1)
      endif
      if (oufile .eq. ' ') then
         write (0, '(a)')  ' Name of output file '
         read (*, '(a)')     oufile
      endif
c
c                         Read header and IJOB
      fname = clfile(1)
      open (ire, file = fname, status = 'old', err = 99)
      rewind ire
           jl = 1
      read (ire, *, err = 98) dname
      print *, 'Data set: ', dname
           jl = jl + 1
      read (ire, *, err = 98) age
           jl = jl + 1
      read (ire, *, err = 98) ncl
      write (0, '(i3, a)') ncl, ' tree class(es)'
           jl = jl + 1
      read (ire, *, err = 98)
           jl = jl + 1
      if (ncl .gt. 1) then
         read (ire, *, err = 98) frefr, (clfile(j), j = 2, ncl)
      else
         read (ire, *, err = 98) frefr
      endif
      do jr = 1, 18
         jl = jl + 1
         read (ire, *, err = 97) cdumm
      enddo
         jl = jl + 1
      read (ire, *, err = 98) lmod3(1)
         jl = jl + 1
      read (ire, *, err = 98) ncomp3(1)
         nl = ncomp3(1) + 13
         if (lmod3(1) .eq. 'liberty') nl = ncomp3(1) + 15
      do jr = 1, nl
         jl = jl + 1
         read (ire, *, err = 98)
      enddo
         jl = jl + 1
      read (ire, *, err = 98) lmod2
         jl = jl + 1
      read (ire, *, err = 98) ncomp2
         nl = ncomp2 + 8
         if (lmod2 .eq. 'liberty') nl = ncomp2 + 10
      do jr = 1, nl
         jl = jl + 1
         read (ire, *, err = 98)
      enddo
      read (ire, *, err = 98) lmod1
      read (ire, *, err = 98) ncomp1
         nl = ncomp1 + 8
         if (lmod1 .eq. 'liberty') nl = ncomp1 + 10
      do jr = 1, nl
         jl = jl + 1
         read (ire, *, err = 98)
      enddo
      read (ire, *, err = 98) ijob
      write (0, '(a, i3, 3a)') 
     & ' ijob = ', ijob, tjob(ijob+1), '    Output file: ', oufile
      close (ire)
c
c                           ! Read forest parameters
      do jcl = 1, ncl
         fname = clfile(jcl)
         open (ire, file = fname, status = 'old', err = 99)
           jl = 6
         read (ire, '(5(/))', err = 98)
           jl = jl + 1
         read (ire, *, err = 98) liik(jcl)
           jl = jl + 1
         read (ire, *, err = 98) lelli(jcl)
         j2 = 14
         do jx = 1, j2
            jl = jl + 1
            if (ijob .gt. 3 .and. jcl .eq. 1) then
               read (ire, *, err = 98) xc(jcl, jx),
     &              xmin(jx), xmax(jx), xdelt(jx)
            else
               read (ire, *, err = 98) xc(jcl, jx)
            endif
         enddo
           jl = jl + 1
         read (ire, *, err = 98) clmpsh(jcl)
           jl = jl + 1
         read (ire, *, err = 98) lmod3(jcl)
           jl = jl + 1
         read (ire, *, err = 98) ncomp3(jcl)
         do jc = 1, ncomp3(jcl)
            jx = j2 + jc
            jl = jl + 1
            if (ijob .gt. 3 .and. jcl .eq. 1) then
               read (ire, *, err = 98) xc(jcl, jx),
     &            xmin(jx), xmax(jx), xdelt(jx), fcoef3(jcl, jc)
            else
               read (ire, *, err = 98) xc(jcl, jx),
     &            dum1, dum2, dum3, fcoef3(jcl, jc)
            endif
         enddo
c
         if (lmod3(jcl) .eq. 'liberty') then
            j1 = 22
            j2 = j1 + 1
            do jx = j1, j2
               if (ijob .gt. 3 .and. jcl .eq. 1) then
                  read (ire, *, err = 98) xc(jcl, jx),
     &                 xmin(jx), xmax(jx), xdelt(jx)
               else
                  read (ire, *, err = 98) xc(jcl, jx)
               endif
            enddo
         endif
c
            j1 = 27
            j2 = j1 + 1
         do jx = j1, j2
            jl = jl + 1
            if (ijob .gt. 3 .and. jcl .eq. 1) then
               read (ire, *, err = 98) xc(jcl, jx),
     &              xmin(jx), xmax(jx), xdelt(jx)
            else
               read (ire, *, err = 98) xc(jcl, jx)
            endif
         enddo
              jl = jl + 1
         read (ire, *, err = 98) brreflf(jcl)
              jl = jl + 1
         read (ire, *, err = 98) trreflf(jcl)
c
         if (jcl .eq. 1) then
              jl = jl + 1
            read (ire, *, err = 98)
c
c                                Read ground vegetation parameters
c                                the upper layer
            j1 = 29
            j2 = j1 + 7
            do jx = j1, j2
               jl = jl + 1
               if (ijob .gt. 3) then
                  read (ire, *, err = 98) xc(jcl, jx),
     &                 xmin(jx), xmax(jx), xdelt(jx)
               else
                  read (ire, *, err = 98) xc(jcl, jx)
               endif
            enddo
              jl = jl + 1
            read (ire, *, err = 98) lmod2
              jl = jl + 1
            read (ire, *, err = 98) ncomp2
            do jc = 1, ncomp2
               jx = j2 + jc
               jl = jl + 1
               read (ire, *, err = 98) xc(jcl, jx),
     &            xmin(jx), xmax(jx), xdelt(jx), fcoef2(jc)
            enddo
            if (lmod2 .eq. 'liberty') then
               j1 = 47
               j2 = j1 + 2
            else
               j1 = 49
               j2 = j1
            endif
            do jx = j1, j2
               jl = jl + 1
               if (ijob .gt. 3) then
                  read (ire, *, err = 98) xc(jcl, jx),
     &              xmin(jx), xmax(jx), xdelt(jx)
               else
                  read (ire, *, err = 98) xc(jcl, jx)
               endif
            enddo
c                                            the lower layer
               j1 = 50
               j2 = j1 + 6
            do jx = j1, j2
               jl = jl + 1
               if (ijob .gt. 3) then
                  read (ire, *, err = 98) xc(jcl, jx),
     &              xmin(jx), xmax(jx), xdelt(jx)
               else
                  read (ire, *, err = 98) xc(jcl, jx)
               endif
            enddo
              jl = jl + 1
            read (ire, *, err = 98) lmod1
              jl = jl + 1
            read (ire, *, err = 98) ncomp1
            do jc = 1, ncomp1
               jx = j2 + jc
               jl = jl + 1
               read (ire, *, err = 98) xc(jcl, jx),
     &            xmin(jx), xmax(jx), xdelt(jx), fcoef1(jc)
            enddo
            if (lmod1 .eq. 'liberty') then
               j1 = 67
               j2 = j1 + 2
            else
               j1 = 69
               j2 = j1
            endif
            do jx = j1, j2
               jl = jl + 1
               if (ijob .gt. 3) then
                  read (ire, *, err = 98) xc(jcl, jx),
     &                 xmin(jx), xmax(jx), xdelt(jx)
               else
                  read (ire, *, err = 98) xc(jcl, jx)
               endif
            enddo
c                                           Read soil parameters
            jl = jl + 1
            read (ire, *, err = 98) fprice, thssr
            j1 = 70
            j2 = j1 + 3
            do jx = j1, j2
               jl = jl + 1
               read (ire, *, err = 98) xc(jcl, jx),
     &              xmin(jx), xmax(jx), xdelt(jx)
            enddo
            thss  = thssr*dr
c
c                                        ! Read 6S parameters
              jl = jl + 1
            read (ire, *, err = 98) iaer, dcaer
              jl = jl + 1
            read (ire, *, err = 98) dvis, dtaer55
c
              jl = jl + 1
            read (ire, *, err = 98) ijob
c
c                           ! Read Sun angles, wavelengths and view angles
              jl = jl + 1
            read (ire, *, err = 98) nsun, nchnl, dwl
            if (ijob .le. 3 .and. ijob .ne. 1) nchnl = 1
            if (ijob .lt. 3) nsun  = 1
            if (ijob .eq. 6) then
               nrefl = nchnl
               nsun  = 1
               nchnl = 1
            endif
            nsangl   = nknots + nsun
              jl = jl + 1
            read (ire, *, err = 98) (dos(jths), jths = 1, nsun)
c
            if (dwl .le. 0.d0) then
              jl = jl + 1
               read (ire, *, err = 98) (wlarr(jw), jw = 1, nchnl)
            else
              jl = jl + 1
               read (ire, *, err = 98) wlarr(1)
            endif
c
              jl = jl + 1
            read (ire, *, err = 98) thetvs, dthtvs, phivs
            if (phivs .gt. 180.d0) phivs = 360.d0 - phivs
c
            if (ijob .gt. 3) then
c                           ! Read minimization parameters
              jl = jl + 1
            read (ire, *, err = 98) mv
              jl = jl + 1
            read (ire, *, err = 98) nfmax, itmax, itbr, nbrak
              jl = jl + 1
            read (ire, *, err = 98) zeps, tol, tiny, ftol
              jl = jl + 1
            read (ire, *, err = 98) alphopt, betopt, gammopt, dxopt
              jl = jl + 1
            read (ire, *, err = 98) nxx, at, lig
c                           ! lig = .true.  - use the previous results 
c                                             as initial guess
              jl = jl + 1
            read (ire, *, err = 98) (llxx(jx), jx = 1, nxx)
c
c
c                               ! Read reflectance/transmittance data
               if (ijob .eq. 6) then
                  js = 1
                  do jr = 1, nrefl
                     read (ire, *, err = 98)
     &               thvr(jr), phir(jr), bqjarr(js, jr)
                  enddo
               else
                  do js = 1, nsun
                     do jw = 1, nchnl
                        jl = jl + 1
                        read (ire, *, err = 98) 
     &                  dum1, bqjarr(js, jw), drhoarr(js, jw)
                        if (abs(dum1 - wlarr(jw)) .gt. eps) then
                           write (0, *)
     &            '*** Warning: Incorrect wavelength: wl = ', dum1,
     &                     '  th_Sun = ', dos(js)
                        endif
                        if (bqjarr(js, jw) .gt. 0.d0) l_struc = .true.
                     enddo
                  enddo
               endif
            endif
c
         endif
         close (ire)
      enddo
c
      do jcl = ncl + 1, nclmax
         ncomp3(jcl) = 0
      enddo
c                                  Read tabulated spectra
      call rspec(frefr, fprice, fcoef1, ncomp1, 
     & fcoef2, ncomp2, fcoef3, ncomp3, ncl)
c
c                                  Read trunk and bark reflectance
      do jcl = 1, ncl
         fname = trreflf(jcl)
         open (ire, file = fname, status = 'old', err = 99)
            rewind ire
            read (ire, *, err = 98) 
     &      (dum1, trrarr(jcl, jl), jl = 1, nspchnl)
         close (ire)
c
         fname = brreflf(jcl)
         open (ire, file = fname, status = 'old', err = 99)
            rewind ire
            read (ire, *, err = 98) 
     &      (dum1, brrarr(jcl, jl), jl = 1, nspchnl)
         close (ire)
      enddo
c
      do jcl = 1, ncl
         if (lelli(jcl)) then
            kuju(jcl) = elli
         else
            kuju(jcl) = koon
         endif
      enddo
c
      open (iwr, file = oufile, status = 'replace')
*     open (iwr, file = oufile, status = 'unknown')
      rewind iwr
c
c                                               ! Print input data
      write (iwr, '(a/2a/2a)') trl, trl, 
     & '  Forest Reflectance Model V.04.2013', trl,
     & '  by A. Kuusk, T. Nilson'
      write (iwr, '(a/a/3a, i4)') trl, 
     & '# Input parameters:', trl, dname, '  Stand Age  = ', age
*
      write (iwr, '(a, i3)') '##  ijob = ', ijob
*
      if (ijob .eq. 0) 
     & write (iwr, '(3(a, f5.1), /a, f5.1, a)') '#   Sun zenith = ', 
     & dos(1), '  View zenith = ', thetvs, ' View azimuth = ', phivs, 
     & '#   Wavelength = ', wlarr(1), ' nm'
c
      if (ijob .eq. 1)
     & write (iwr, '(3(a, f5.1), /a, i6, a)') '#   Sun zenith = ', 
     & dos(1), '  View zenith = ', thetvs, '  View azimuth = ', phivs,
     & trl, nchnl, ' spectral bands'
c
      if (ijob .eq. 2)
     & write (iwr, '(3(a, f5.1), /a, f5.1, a)') '#   Sun zenith = ', 
     & dos(1), ' View azimuth = ', phivs, '  View zenith step = ', 
     & dthtvs, '#   Wavelength = ', wlarr(1), ' nm'
c
      if (ijob .eq. 3)
     &  write (iwr, '(3(a, f6.1), a/a, 9f6.1)') '#   View zenith = ', 
     & thetvs, '   View azimuth = ', phivs, '   Wavelength = ', 
     & wlarr(1), ' nm', '#   Sun angles = ', (dos(jths), jths = 1, nsun)
c
      if (ijob .eq. 4)
     & write (iwr, '(a)') '#   Model inversion, relative differences'
c
      if (ijob .eq. 5)
     & write (iwr, '(a)') '#   Model inversion, absolute differences'
c
      if (ijob .eq. 6)
     & write (iwr, '(a)') '#   Model inversion, several view directions'
c
        write (iwr, '(a, i3, a)') trl,
     & ncl, ' tree class(es)'
      if (ncl .gt. 1) write (iwr, '(a46 / a, 12(2x, a16))') 
     & '#   Files of parameters of other tree classes:',
     & trl, (clfile(jcl), jcl = 2, ncl)
c
      if (ijob .gt. 3) then
         write (iwr, '(a, 48x, 25(i4, 4x))') trl,  (i, i = 1, ncl)
         write (iwr, '(a, 27x, 22a8)') trl, 'xmin  ', 
     &    '  xmax  ', '  dx      ', (liik(jcl), jcl = 1, ncl)
         write (iwr, '(a, 49x, 20a8)') 
     &    trl, (kuju(jcl), jcl = 1, ncl)
      else
         write (iwr, '(a, 24x, 25(i4, 4x))') trl,  (i, i = 1, ncl)
         write (iwr, '(a, 27x, 22a8)') trl,
     &    (liik(jcl), jcl = 1, ncl)
         write (iwr, '(a, 25x, 20a8)') 
     &    trl, (kuju(jcl), jcl = 1, ncl)
      endif
c
      jx = 1
      if (ijob .gt. 3) then
         write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl,
     & jx, tex(jx), xmin(jx), xmax(jx), xdelt(jx),
     & (xc(jcl, jx),  jcl = 1, ncl)
      else
         write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, 
     & jx, tex(jx), (xc(jcl, jx),  jcl = 1, ncl)
      endif
      j2 = 14
      do jx = 2, j2
         if (ijob .gt. 3) then
            write (iwr, '(a, i3, 1x, a20, 28f8.3)') trl, 
     & jx, tex(jx), xmin(jx), xmax(jx), xdelt(jx), 
     & (xc(jcl, jx),  jcl = 1, ncl)
         else
            write (iwr, '(a, i3, 1x, a20, 28f8.3)') trl, 
     & jx, tex(jx), (xc(jcl, jx),  jcl = 1, ncl)
         endif
      enddo
c
      if (ijob .le. 3) then
         write (iwr, '(a, 1x, a16, 7x, 10f8.3)') 
     &    trl, 'Shoot clumping:  ', (clmpsh(jcl), jcl = 1, ncl)
      else
         write (iwr, '(a, 1x, a16, 31x, 10f8.3)') 
     &    trl, 'Shoot clumping:  ', (clmpsh(jcl), jcl = 1, ncl)
      endif
      write (iwr, '(a, 1x, a14, 2x, 10a18)') 
     & trl, 'Leaf models: ', (lmod3(jcl), jcl = 1, ncl)
      if (ijob .gt. 3) then
         write (iwr, '(a, 2x, a18, 26x, 10i8)')
     & trl, '# of leaf comp-s:', (ncomp3(jcl), jcl = 1, ncl)
      else
         write (iwr, '(a, 2x, a17, 2x, 10i8)')
     & trl, '# of leaf comp-s:', (ncomp3(jcl), jcl = 1, ncl)
      endif
      ncmax = 1
      do jc = 1, ncl
         if (ncomp3(jc) .gt. ncmax) ncmax = ncomp3(jc)
      enddo
      do jc = 1, ncmax
         jx = j2 + jc
         if (ijob .gt. 3) then
            write (iwr, '(a, 5x, 10a18)')
     & trl, (fcoef3(jcl, jc), jcl = 1, ncl)
            write (iwr, '(a, i3, 1x, a20, 28f8.2)') trl, 
     & jx, tex(jx), xmin(jx), xmax(jx), xdelt(jx),
     & (xc(jcl, jx), jcl = 1, ncl)
         else
            write (iwr, '(a, 5x, 10a18)')
     & trl, (fcoef3(jcl, jc), jcl = 1, ncl)
            write (iwr, '(a, i3, 1x, a20, 28f8.2)') trl, 
     & jx, tex(jx), (xc(jcl, jx), jcl = 1, ncl)
         endif
      enddo
c
      lml = .false.
      do jcl = 1, ncl
         if (lmod3(jcl) .eq. 'liberty') then
            lml = .true.
            goto 33
         endif
      enddo
33    continue
      if (lml) then
         j1 = 25
         j2 = j1 + 1
         do jx = j1, j2
            if (ijob .gt. 3) then
               write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, jx, 
     & tex(jx), xmin(jx), xmax(jx), xdelt(jx),
     & (xc(jcl, jx),  jcl = 1, ncl)
            else
               write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, jx, 
     &       tex(jx), (xc(jcl, jx),  jcl = 1, ncl)
            endif
         enddo
      endif
c
      jx = 27
        if (ijob .gt. 3) then
            write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, jx, 
     & tex(jx), xmin(jx), xmax(jx), xdelt(jx),
     & (xc(jcl, jx),  jcl = 1, ncl)
         else
            write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, jx, 
     & tex(jx), (xc(jcl, jx),  jcl = 1, ncl)
         endif
c
      jx = 28
        if (ijob .gt. 3) then
            write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, jx, 
     & tex(jx), xmin(jx), xmax(jx), xdelt(jx),
     & (xc(jcl, jx),  jcl = 1, ncl)
         else
            write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, jx, 
     & tex(jx), (xc(jcl, jx),  jcl = 1, ncl)
         endif
c
      write (iwr, '(a21, 12(2x, a24))') '#   bark refl. files: ',
     & (brreflf(jcl), jcl = 1, ncl)
      write (iwr, '(a21, 12(2x, a24))') '#   trunk refl. files:',
     & (trreflf(jcl), jcl = 1, ncl)
c
      write (iwr, '(a/a)') trl, '# *** Ground vegetation, upper layer'
      j2 = 8
      jt = 28
      do jr = 1, j2
         jx = jx + 1
         jt = jt + 1
         if (ijob .gt. 3) then
            write (iwr, '(a, i3, 1x, a20, 28f8.2)') trl, 
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx),
     & xc(1, jx)
         else
            write (iwr, '(a, i3, 1x, a20, 28f8.2)') trl, 
     & jx, tex(jt), xc(1, jx)
         endif
      enddo
c
      write (iwr, '(a, 2x, a12, 2x, a12)') 
     & trl, 'Leaf model: ', lmod2
      write (iwr, '(a, 2x, a21, i6)')
     & trl, '# of leaf components:', ncomp2
      jt = 14
      do jc = 1, ncomp2
           jx = jx + 1
           jt = jt + 1
         if (ijob .gt. 3) then
            write (iwr, '(a, 5x, 10a18)')
     & trl, fcoef2(jc)
            write (iwr, '(a, i3, 1x, a20, 28f8.2)') trl, 
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx),
     & xc(1, jx)
         else
            write (iwr, '(a, 5x, 10a18)')
     & trl, fcoef2(jc)
            write (iwr, '(a, i3, 1x, a20, 28f8.2)') trl, 
     & jx, tex(jt), xc(1, jx)
         endif
      enddo
c
      if (lmod2 .eq. 'liberty') then
         jt = 23
         j1 = 47
         j2 = j1 + 1
         do jx = j1, j2
            jt = jt + 1
            if (ijob .gt. 3) then
               write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, 
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx), xc(1, jx)
            else
               write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, 
     & jx, tex(jt), xc(1, jx)*100.d0
            endif
         enddo
      else
         jt = 27
         jx = 49
         if (ijob .gt. 3) then
            write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, 
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx), xc(1, jx)
         else
            write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, 
     & jx, tex(jt), xc(1, jx)
         endif
      endif
c
      write (iwr, '(a)') '# *** Ground vegetation, lower layer'
      j1 = 50
      j2 = j1 + 6
      jt = 28
      do jx = j1, j2
         jt = jt + 1
         if (jt .eq. 32) jt = jt + 1
         if (ijob .gt. 3) then
            write (iwr, '(a, i3, 1x, a20, 28f8.2)') trl, 
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx),
     & xc(1, jx)
         else
            write (iwr, '(a, i3, 1x, a20, 28f8.2)') trl, 
     & jx, tex(jt), xc(1, jx)
         endif
      enddo
c
      write (iwr, '(a, 2x, a12, 2x, a12)') 
     & trl, 'Leaf model: ', lmod1
      write (iwr, '(a, 2x, a21, i6)')
     & trl, '# of leaf components:', ncomp1
      jt = 14
      jx = j2
      do jc = 1, ncomp1
           jx = jx + 1
           jt = jt + 1
         if (ijob .gt. 3) then
            write (iwr, '(a, 5x, 10a18)') trl, fcoef1(jc)
            write (iwr, '(a, i3, 1x, a20, 28f8.2)') trl, 
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx), xc(1, jx)
         else
            write (iwr, '(a, 5x, 10a18)') trl, fcoef1(jc)
            write (iwr, '(a, i3, 1x, a20, 28f8.2)') trl, 
     & jx, tex(jt), xc(1, jx)
         endif
      enddo
c
      if (lmod1 .eq. 'liberty') then
         j1 = 67
         jt = 24
         j2 = j1 + 1
         do jx = j1, j2
            jt = jt + 1
            if (ijob .gt. 3) then
               write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, 
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx), 
     &          xc(1, jx)
            else
               write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, 
     &          jx, tex(jt), xc(1, jx)
            endif
         enddo
      endif
         jx = 69
         jt = 27
         if (ijob .gt. 3) then
            write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, jx, 
     & tex(jt), xmin(jx), xmax(jx), xdelt(jx), xc(1, jx)
         else
            write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, 
     & jx, tex(jt), xc(1, jx)
         endif
c
      jt = 36
      j1 = 70
      j2 = j1 + 3
      do jx = j1, j2
         jt = jt + 1
         if (ijob .gt. 3) then
            write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, 
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx), xc(1, jx)
         else
            write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, 
     & jx, tex(jt), xc(1, jx)
         endif
      enddo
c
      if (ijob .gt. 3) then
         write (iwr, '(a/a, i6, 3(a, i4), 2(/(a, 4g10.2)))') trl, 
     &    '# nfmax = ', nfmax, '      itmax = ', itmax, 
     &    '     itbr  = ', itbr, '      nbrak = ', nbrak,
     &    '# zeps, tolbr, tiny, ftolp: ', zeps, tol, tiny, ftol,
     &    '# alpha, beta, gamma, dx: ', alphopt, betopt, gammopt, dxopt
         if (lig) then
            write (iwr, '(a, g8.3, a)') '# at  = ', at, 'lx = TRUE'
         else
            write (iwr, '(a, g8.3, a)') '# at  = ', at, 'lx = FALSE'
         endif
         write (iwr, '(a, i3, 3x, a, 15i4)') 
     &         '# n   = ', nxx, '  ll = ', (llxx(i), i = 1, nxx)
      endif
c                  !  print input reflectances and transmittances
c
      if (ijob .eq. 4 .or. ijob .eq. 5) then
         write (iwr, '(a/a, f5.1, a, f6.1/a/a)') trl, 
     &     '# thv = ', thetvs, '   phi = ', phivs, 
     &     '# Input reflectance', 
     &     '#       th_sun   lambda    refl.    d_rho'
         do jths = 1, nsun
            write (iwr, '(a, f10.1, f10.0, 2f10.4)') trl, 
     &          dos(jths), wlarr(1), 
     &          bqjarr(jths, 1), drhoarr(jths, 1)
            if (nchnl .gt. 1) 
     &         write (iwr, '(a, 10x, f10.0, 2f10.4)') (trl, wlarr(jw), 
     &         bqjarr(jths, jw), drhoarr(jths, jw), jw = 2, nchnl)
         enddo
      endif
c
      if (ijob .eq. 6) then
         write (iwr, '(a/a, f5.1, 4x, a, f7.1/a/a)') trl, 
     &     '# Sun zenith angle', dos(1), 'Spectral band', wlarr(1),
     &     '# Input reflectance',
     &     '#    th_view  phi_view    refl.'
         do jr = 1, nrefl
            write (iwr, '(a, 2f10.1, f10.4)') trl,
     &          thvr(jr), phir(jr), bqjarr(1, jr)
         enddo
      endif
c
      write (iwr, '(a)') '#  '
c
c
c  ******************                    ! Start of computations
c
c                                        ! Check wavelength range
      do jw = 1, nchnl
         if (dwl .gt. 0.d0) wlarr(jw) = wlarr(1) + (jw - 1)*dwl
         rlambda   = wlarr(jw)
         if ((rlambda .lt. 400.d0) .or. (rlambda .gt. 2400.d0)) 
     &     goto 88
      enddo
c
c                                            ! Check crown length
      do jcl = 1, ncl
         if (lelli(jcl)) then
            if (xc(jcl, 3) .gt. xc(jcl, 2)) then
               xc(jcl, 3) = xc(jcl, 2)
          print *, 'jcl = ', jcl
          print *, 'Warning: tree height is less than crown length,'
               print *, 'the crown length is adjusted.'
            endif
         else
            hcrown = xc(jcl, 3) + xc(jcl, 4)
            if (xc(jcl, 2) .lt. xc(jcl, 3)) then
               xc(jcl, 3) = xc(jcl, 2)
               xc(jcl, 4) = 0.d0
          print *, 'jcl = ', jcl
          print *, 'Warning: tree height is less than crown length,'
               print *, 'the crown length is adjusted.'
            else if (hcrown .gt. xc(jcl, 2)) then
               xc(jcl, 4) = xc(jcl, 2) - xc(jcl, 3)
          print *, 'jcl = ', jcl
          print *, 'Warning: tree height is less than crown length,'
               print *, 'the crown length is adjusted.'
            endif
         endif
      enddo
c
      thknot(1) = thetvs*dr
c                               ! Sun zenith angles into the array
c                               ! of Gauss quadrature knots
      do jths = 1, nsun
         i         = nknots + jths
         thknot(i) = dos(jths)*dr
      enddo
c                                              ! S/D ratio (6S)
      if (iaer .eq. -1) then
         write (iwr, '(a)') '#  '
         write (iwr, '(a)') '#  No sky radiation   (BDRF)'
         write (iwr, '(a)') '#  '
      else
         write (iwr, '(a)') '# 6S parameters'
      endif
      do ii1 = 2, nsangl
         if (iaer .eq. -1) then
            do jw = 1, nchnl
               jwl = nint((wlarr(jw) - (400.d0 - wlstep))/wlstep)
               sqarr(ii1, jwl) = 1.d0
            enddo
         else
            call sixd
     &          (iwr, nchnl, wlarr, wlstep, ii1, thknot,
     &           iaer, dcaer, dvis, dtaer55, sqarr)
         endif
      enddo
c
c                            ! copy parameters into common block
      l_opt   = .true.
      l_struc = .true.
      i_job   = ijob
      n_sangl = nsangl
      n_refl  = nrefl
c
      thetv   = thetvs*dr
      p_hi = phivs*dr
      if (abs(thetv) .lt. dr) then
         p_hi = 0.d0
      endif
      phi = p_hi
      p_khair = pkhair
      n_sun   = nsun
      if (ijob .eq. 1) then
         n_chnl = 1
      else
         n_chnl = nchnl
      endif
      l_mod1  = lmod1
      l_mod2  = lmod2
      n_cmp1  = ncomp1
      n_cmp2  = ncomp2
      n_cl    = ncl
      n_zs    = nzs
      t_hss   = thss
      d_cel0  = dcell0
      a_t     = at
c
      ntstz   = 2*nzs
      x1      = -1.d0
      x2      = 1.d0
      call gauleg(x1, x2, zctst, acztst, ntstz)
c
      do jcl = 1, ncl
         l_elli(jcl)  = lelli(jcl)
         c_lmpsh(jcl) = clmpsh(jcl)
         l_mod3(jcl)  = lmod3(jcl)
         n_cmp3(jcl)  = ncomp3(jcl)
         do jw = 1, nspchnl
            b_rrarr(jcl, jw) = brrarr(jcl, jw)
            t_rrarr(jcl, jw) = trrarr(jcl, jw)
         enddo
         do jx = 1, nvar
            x_c(jcl, jx)     = xc(jcl, jx)
         enddo
      enddo
c
      do jw = 1, nspchnl
         w_larr(jw)  = wlarr(jw)
      enddo
c
      do jths = 1, nsangl
         do jw = 1, nspchnl
            s_qarr(jths, jw)  = sqarr(jths, jw)
         enddo
         t_hknot(jths)  = thknot(jths)
      enddo
      do jths = 1, nknots
         p_14(jths)      = p14(jths)
      enddo
c
c                            initialize the vector of parameters
      do jx = 1, nvar
         xx(jx) = xc(1, jx)
      enddo
c                             the parameters of ground model
      clmp2 = xx(31)
      ul2   = xx(29)*clmp2
      sl2   = xx(30)
      szz   = xx(32)
      eln2  = xx(33)
      thm2  = xx(34)*dr
      rnc2  = xx(35)
      slw2  = xx(36)
      jx    = 36
      do jc = 1, ncomp2
         jx = jx + 1
         ccomp2(jc) = xx(jx)
      enddo
      dcell2 = xx(47)
      asp2  = xx(48)
      vai2  = xx(49)/clmp2
      clmp1 = xx(52)
      ul1   = xx(50)*clmp1
      sl1   = xx(51)
      eln1  = xx(53)
      thm1  = xx(54)*dr
      rnc1  = xx(55)
      slw1  = xx(56)
      jx    = 56
      do jc = 1, ncomp1
         jx = jx + 1
         ccomp1(jc) = xx(jx)
      enddo
      dcell1 = xx(67)
      asp1  = xx(68)
      vai1  = xx(69)/clmp1
      rsl1  = xx(70)
      rsl2  = xx(71)
      rsl3  = xx(72)
      rsl4  = xx(73)
c
c                            ! copy parameters into common block(s)
      do jx = 1, nvar
         l_lxx(jx)  = llxx(jx)
         x_0(jx)    = xx(jx)
         x_x(jx)    = xx(jx)
         x_delt(jx) = xdelt(jx)
      enddo
c
c
c                ******************  ijob = 0, the single direct run
c
      if (ijob .eq. 0) then
         fff  = func(nxx, xxf)
c
         jv = 1
         jw = 1
         js = 1
         jwl = nint((wlarr(jw) - (400.d0 - wlstep))/wlstep)
         thets = dos(js)*dr
         skyl  = 1.d0 - sqarr(nknots+js, jwl)
            call smcrm
     &  (thknot, p14, 
     &  lmod2, ncomp2, ul2, sl2, szz, eln2, thm2, 
     &  slw2, rnc2, vai2, dcell2, asp2, ccomp2, 
     &  lmod1, ncomp1, ul1, sl1, eln1, thm1, 
     &  slw1, rnc1, vai1, dcell1, asp1, ccomp1, 
     &  thets, bangs, pkhair, dcell0, rsl1, rsl2, rsl3, rsl4, thss,
     &  thetv, phi, rlambda, skyl, ee1, rn1, refl1, tran1, 
     &  ee2, rn2, refl2, tran2, rsl,
     &  ggv, penalty, bc1, bs1, bdplants, bdsoil)
            sbgr(js) = (1 - skyl)*(bc1 + bs1) + bdplants + bdsoil
            sq(js) = 1.d0 - skyl
c
         goto 771
      endif
c
c
c                ******************  Loop to calculate spectrum
c
      if (ijob .eq. 1) then
         jw  = 0
         js  = 1
         jp  = 1
         write (0, '(3(a, f4.1), i8, a)') 
     &       ' th_Sun = ', dos(js), '    thv = ', thetvs, 
     &       '    phi = ', phivs, nchnl, ' spectral bands'
         thetv   = thetvs*dr
         thvhm = abs(thetv)
      endif
c
c
550      continue
      if (ijob .eq. 1) then
         jw = jw + 1
         rlambda    = wlarr(jw)
         w_larr(1)  = rlambda
c
         fff  = func(nxx, xxf)
c
         jwl = nint((wlarr(jw) - (400.d0 - wlstep))/wlstep)
         thets = dos(js)*dr
         skyl  = 1.d0 - sqarr(nknots+js, jwl)
         call smcrm
     &  (thknot, p14, 
     &  lmod2, ncomp2, ul2, sl2, szz, eln2, thm2, 
     &  slw2, rnc2, vai2, dcell2, asp2, ccomp2, 
     &  lmod1, ncomp1, ul1, sl1, eln1, thm1, 
     &  slw1, rnc1, vai1, dcell1, asp1, ccomp1, 
     &  thets, bangs, pkhair, dcell0, rsl1, rsl2, rsl3, rsl4, thss,
     &  thvhm, phi, rlambda, skyl, ee1, rn1, refl1, tran1, 
     &  ee2, rn2, refl2, tran2, rsl,
     &  ggv, penalty, bc1, bs1, bdplants, bdsoil)
         sbgr(js) = (1 - skyl)*(bc1 + bs1) + bdplants + bdsoil
         sq(js) = 1.d0 - skyl
c
         goto 771
      endif
c
c
c                ******************  Loop to calculate angular distribution
c
      if (ijob .eq. 2) then
         js  = 1
         jw  = 1
         jwl = nint((wlarr(jw) - (400.d0 - wlstep))/wlstep)
         thtv00 = 80.d0
         nview1  = int(thtv00/dthtvs)
         nview   = 2*nview1 + 1
         if (abs(phi - pi/2.d0) .lt. eps) then
            nv = nview1 + 1
         else
            nv = nview
         endif
         dthetv = dthtvs*dr
         thtv0  = -(thtv00/dthtvs + 1)*dthetv
         write (0, '(2(a, f4.1), a, f6.1, i8, a)')
     &       ' th_Sun = ', dos(1), '    phi = ', phivs, 
     &       '   Spectral band ', rlambda, nview, ' view angles'
c
         do jv = 1, nv
            thetv      = thtv0 + jv*dthetv
            if (abs(thetv) .lt. eps2) then
               thetv   = 0.d0
               p_hi    = 0.d0
            else
               p_hi    = phivs*dr
            endif
            if (jv .le. nview1) then
               thetv = abs(thetv)
               p_hi = phivs*dr
            else
               p_hi   = pi - phivs*dr
            endif
c                            ! view angle is thknot(1)
            thvhm = abs(thetv)
            t_hknot(1) = thvhm
            phi        = p_hi
            l_struc    = .true.
c
            fff  = func(nxx, xxf)
c
            thets = dos(js)*dr
            skyl  = 1.d0 - sqarr(nknots+js, jwl)
            call smcrm
     &  (thknot, p14,
     &  lmod2, ncomp2, ul2, sl2, szz, eln2, thm2,
     &  slw2, rnc2, vai2, dcell2, asp2, ccomp2,
     &  lmod1, ncomp1, ul1, sl1, eln1, thm1,
     &  slw1, rnc1, vai1, dcell1, asp1, ccomp1,
     &  thets, bangs, pkhair, dcell0, rsl1, rsl2, rsl3, rsl4, thss,
     &  thvhm, phi, rlambda, skyl, ee1, rn1, refl1, tran1,
     &  ee2, rn2, refl2, tran2, rsl,
     &  ggv, penalty, bc1, bs1, bdplants, bdsoil)
            sbgr(js) = (1 - skyl)*(bc1 + bs1) + bdplants + bdsoil
            sq(js) = 1.d0 - skyl
c
            rthet(jv)    = p_2bq(1)
            rsthet(jv)   = sbgr(1)
            bdown(jv)    = b_down(1)
            aavth(jv)    = a_av
            rcr1(jv)     = b_qc1(1)
            rgr1(jv)     = b_qg1(1)
            rdif(jv)     = b_qm(1)
            if (abs(phi - pi/2.d0) .lt. eps) then
               jv2 = nview - jv
               rthet(jv2)    = rthet(jv)
               rsthet(jv2)   = rsthet(jv)
               bdown(jv2)    = bdown(jv)
               aavth(jv2)    = aavth(jv)
               rcr1(jv2)     = rcr1(jv)
               rgr1(jv2)     = rgr1(jv)
               rdif(jv2)     = rdif(jv)
            endif
*           print *, thetv/dr, p_2bq(1), b_down(1)
         enddo
         goto 771
      endif
c
c
c                ******************  Several Sun zenith angles
c
      if (ijob .eq. 3) then
         jw  = 1
         jwl = nint((wlarr(jw) - (400.d0 - wlstep))/wlstep)
c                            ! view angle is thknot(1)
         thvhm = abs(thetv)
         t_hknot(1) = thvhm
         phi        = p_hi
         l_struc    = .true.
c
         do js = 1, nsun
c
            fff  = func(nxx, xxf)
c
            thets = dos(js)*dr
            skyl  = 1.d0 - sqarr(nknots+js, jwl)
            call smcrm
     &  (thknot, p14,
     &  lmod2, ncomp2, ul2, sl2, szz, eln2, thm2,
     &  slw2, rnc2, vai2, dcell2, asp2, ccomp2,
     &  lmod1, ncomp1, ul1, sl1, eln1, thm1,
     &  slw1, rnc1, vai1, dcell1, asp1, ccomp1,
     &  thets, bangs, pkhair, dcell0, rsl1, rsl2, rsl3, rsl4, thss,
     &  thvhm, phi, rlambda, skyl, ee1, rn1, refl1, tran1,
     &  ee2, rn2, refl2, tran2, rsl,
     &  ggv, penalty, bc1, bs1, bdplants, bdsoil)
            sbgr(js) = (1 - skyl)*(bc1 + bs1) + bdplants + bdsoil
            sq(js) = 1.d0 - skyl
         enddo
         goto 771
      endif
c
c
c                                ********* Inversion ********
c
      if (ijob .eq. 4 .or. ijob .eq. 5) then
         do jths = 1, nsangl
            do jw = 1, nspchnl
               b_qjarr(jths, jw)  = bqjarr(jths, jw)
               d_rhoarr(jths, jw) = drhoarr(jths, jw)
            enddo
         enddo
      endif
c
      if (ijob .eq. 6) then
         jths = 1
         do jr = 1, nrefl
            t_hvr(jr)          = thvr(jr)*dr
            p_hir(jr)          = (180.d0 - abs(phir(jr)))*dr
            b_qjarr(jths, jr)  = bqjarr(jths, jr)
            d_rhoarr(jths, jr) = bqjarr(jths, jr)*0.1d0
         enddo
      endif
c
      ipass  = 0
      xfs    = oufile
      outx   = xfs//'0.dat'
      if (lig) then
c                          ! read the results of previous run
         jl = 1
         fname = outx
         open (7, file = fname, status = 'old', err = 99)
         rewind 7
         read (7, '(d27.17)', err = 98) (xx(jx), jx = 1, nvar)
         close (7)
      endif
c
      fname = 'flow.dat'
      open (4, file = fname, status = 'old', err = 99)
      rewind 4
      jl = 1
c
400   continue
         i_job = ijob
c
c                            scale the vector of parameters
      do i = 1, nxx
         li        = llxx(i)
         xa(li)    = (xmax(li) + xmin(li))*.5d0
         xb(li)    = 10.d0/(xmax(li) - xmin(li))
         xxf(i)    = (xx(li) - xa(li))*xb(li)
         if (li .le. 10) then
            l_struc = .true.
         endif
      enddo
c
c                            ! copy parameters into common block(s)
      do jx = 1, nvar
         l_lxx(jx)  = llxx(jx)
         x_0(jx)    = xx(jx)
         x_x(jx)    = xx(jx)
         x_a(jx)    = xa(jx)
         x_b(jx)    = xb(jx)
         x_delt(jx) = xdelt(jx)
      enddo
c
      jv = 1
      jw = 1
c
c
      fff  = func(nxx, xxf)
c
      jwl = nint((wlarr(jw) - (400.d0 - wlstep))/wlstep)
      do is = 1, nsun
         thets = dos(is)*dr
         skyl  = 1.d0 - sqarr(nknots+is, jwl)
         call smcrm
     &  (thknot, p14,
     &  lmod2, ncomp2, ul2, sl2, szz, eln2, thm2,
     &  slw2, rnc2, vai2, dcell2, asp2, ccomp2,
     &  lmod1, ncomp1, ul1, sl1, eln1, thm1,
     &  slw1, rnc1, vai1, dcell1, asp1, ccomp1,
     &  thets, bangs, pkhair, dcell0, rsl1, rsl2, rsl3, rsl4, thss,
     &  thetv, phi, rlambda, skyl, ee1, rn1, refl1, tran1,
     &  ee2, rn2, refl2, tran2, rsl,
     &  ggv, penalty, bc1, bs1, bdplants, bdsoil)
         sbgr(is) = (1 - skyl)*(bc1 + bs1) + bdplants + bdsoil
         sq(is) = 1.d0 - skyl
         enddo
c
c
      ipass = ipass + 1
      write (iwr, '(a/a/a/a, i3/a)') trl, trl, 
     &    '########################', '#  Results,     pass', ipass,
     &    '########################'
      write (iwr, '(a/a, g20.14)') trl, '#  f(x0) = ', fff
      write (*, '(/a, e20.14, a, i4, a, i5, a, i3)') 
     &    '  f = ', fff, '   iter = ', iter, 
     &    '   ncount = ', ncount, '  ier = ', ier
      ier = 0
      close (iwr)
c
c                                    ***     powell
      if (mv .eq. 'powell') then
         do i = 1, nxx
            do j = 1, nxx
               if (i .eq. j) then
                  parr(j, i) = dxopt
               else
                  parr(j, i) = 0.d0
               endif
            enddo
         enddo
c
         call powell(xxf, parr, nxx, ftol, fff, iter)
c
         do i = 1, nxx
            li     = llxx(i)
            xx(li) = xxf(i)/xb(li) + xa(li)
         enddo
      endif
      goto 772
c
**************************
**************************
c
c
c                               ! Output results, direct run
c
771   continue
      if (ijob .eq. 1 .and. jp .eq. 2) goto 551
c
      write (iwr, '(a/a)') trl, '# *** Results:'
c
      write (iwr, '(a, 24x, 27(i4, 6x))') trl, (jcl, jcl = 1, ncl)
      write (iwr, '(a, 25x, 27a10)') trl, (liik(jcl), jcl = 1, ncl), 
     &         'totals    '
      write (iwr, '(a, 23x, 25a10)') trl, (kuju(jcl), jcl = 1, ncl)
c
      jx = 0
      j2 = 14
      do j = 1, j2
         jx = jx + 1
            write (iwr, '(a, 1x, a20, 28f10.3)')
     & trl, tex(jx), (xc(jcl, jx),  jcl = 1, ncl), x_m(jx)
      enddo
c
      write (iwr, '(a, 1x, a16, 4x, 15f10.3)')
     & trl, 'Shoot clumping:  ', (clmpsh(jcl), jcl = 1, ncl)
      write (iwr, '(a, 1x, a13, 2x, 15a12)')
     & trl, 'Leaf models: ', (lmod3(jcl), jcl = 1, ncl)
      write (iwr, '(a, 1x, a17, 15i10)')
     & trl, '# of leaf comp-s:', (ncomp3(jcl), jcl = 1, ncl)
      do jc = 1, ncmax
         jx = jx + 1
            write (iwr, '(a, 5x, 15a18)')
     & trl, (fcoef3(jcl, jc), jcl = 1, ncl)
            write (iwr, '(a, 1x, a20, 28f10.2)')
     & trl, tex(jx), (xc(jcl, jx), jcl = 1, ncl)
      enddo
c
      if (lml) then
         j1 = 25
         j2 = j1 + 1
         do j = j1, j2
            jt = j
            write (iwr, '(a, 1x, a20, 28f10.4)')
     &    trl, tex(jt), (xc(jcl, jt),  jcl = 1, ncl)
         enddo
      endif
         jx = 27
         write (iwr, '(a, 1x, a20, 28f10.4)') trl, 
     & tex(jx), (xc(jcl, jx),  jcl = 1, ncl)
c
      write (iwr, '(a21, 17(2x, a24))') '#   bark refl. files: ',
     & (brreflf(jcl), jcl = 1, ncl)
      write (iwr, '(a21, 17(2x, a24))') '#   trunk refl. files:',
     & (trreflf(jcl), jcl = 1, ncl)
      write (iwr, '(4(a, f7.4))') '#   rl_eff = ', r_leff, 
     & '  tl_eff = ', t_leff, '  n_eff = ', r_neff, '  rsl = ', rsl
      write (iwr, '(a, 1x, a19, 1x, 26f10.3)') trl,
     & 'leaf area density ', (u_uu(jcl), jcl = 1, ncl)
      write (iwr, '(a, 2x, a9, 2x, f8.3)')
     & trl, 'Total LAI', t_lai
      write (iwr, '(a, 2x, a9, 2x, f8.3)')
     & trl, 'Total BAI', x_m(12)
      write (iwr, '(a, f6.3, 8x, a, f6.3)') 
     & '#   crown closure = ', v_liit, 'canopy closure = ', c_ano
      if ( jwcl .gt. 0) then
        write (*, '(/a)') 
     &    '*** Warning!: canopy closure (CC) > crown closure (CR)'
        write (*, '(a, i4, 2g12.4/)') 
     &    '          class, CR, CC:', jwcl, wvliit, wcano
        write (iwr, '(a/a)') trl,
     &    '# Warning!: canopy closure (CC) > crown closure (CR)'
        write (iwr, '(a, i4, 2g12.4/a)') 
     &    '#           class, CR, CC:', jwcl, wvliit, wcano, trl
      endif
c
      if (ijob .eq. 0) then
         write (iwr, '(a)') '#        z          uz'
         do jz = 1, 400
            if (u_uz(jz) .gt. 0.d0) write (iwr, '(a, 2f12.4)') 
     &    trl, jz*0.1d0, u_uz(jz)
         enddo
      endif
c
      write (iwr, '(a/a)') trl, '#  *** Ground vegetation, upper layer'
      j2 = 8
      jx = 28
      do j = 1, j2
         jx = jx + 1
         write (iwr, '(a, 1x, a20, 28f10.2)')
     & trl, tex(jx), xc(1, jx)
      enddo
c
      write (iwr, '(a, 2x, a12, 2x, 15a8)') 
     & trl, 'Leaf model: ', lmod2
      write (iwr, '(a, 2x, a21, i8)')
     & trl, '# of leaf components:', ncomp2
      jt = 14
      do jc = 1, ncomp2
         jx = jx + 1
         jt = jt + 1
         write (iwr, '(a, 5x, 15a18)') trl, fcoef2(jc)
         write (iwr, '(a, 1x, a20, 28f10.2)')
     & trl, tex(jt), xc(1, jx)
      enddo
c
      if (lmod2 .eq. 'liberty') then
         jt = 23
         j1 = 44
         j2 = j1 + 1
         do jx = j1, j2
            jt = jt + 1
            write (iwr, '(a, 1x, a20, 28f10.2)')
     &    trl, tex(jt), xc(1, jx)
         enddo
      endif
         jt = 27
         jx = 49
         write (iwr, '(a, 3x, a20, 28f8.4)')
     & trl, tex(jt), xc(1, jx)
c
      write (iwr, '(a)') '#  *** Ground vegetation, lower layer'
      j1 = 50
      j2 = j1 + 6
      jt = 28
      do jx = j1, j2
         jt = jt + 1
            write (iwr, '(a, 1x, a20, 28f10.2)')
     & trl, tex(jt), xc(1, jx)
      enddo
c
      write (iwr, '(a, 2x, a12, 2x, 15a8)') 
     & trl, 'Leaf model: ', lmod1
      write (iwr, '(a, 2x, a21, i8)')
     & trl, '# of leaf components:', ncomp1
      jt = 14
      jx = j2
      do jc = 1, ncomp1
         jx = jx + 1
         jt = jt + 1
         write (iwr, '(a, 5x, 15a18)') trl, fcoef1(jc)
         write (iwr, '(a, 1x, a20, 28f10.2)')
     & trl, tex(jt), xc(1, jx)
      enddo
c
      if (lmod1 .eq. 'liberty') then
         j1 = 67
         jt = 24
         j2 = j1 + 1
         do jx = j1, j2
            jt = jt + 1
            write (iwr, '(a, 1x, a20, 28f10.4)')
     &    trl, tex(jt), xc(1, jx)
         enddo
      endif
         jx = 69
         jt = 27
         write (iwr, '(a, 1x, a20, 28f10.4)')
     & trl, tex(jt), xc(1, jx)
c
      jt = 36
      j1 = 70
      j2 = j1 + 3
      do jx = j1, j2
         jt = jt + 1
         write (iwr, '(a, 1x, a20, 28f10.4)')
     & trl, tex(jt), xc(1, jx)
      enddo
c
c                               Output results
c
      if (ijob .eq. 0) then
         write (0, '(3(a, f5.1), a, f7.1)') 
     &      ' th_s = ', dos(1), '    th_view = ', thetvs,
     &      '    phi = ', phivs, '    Spectral band ', rlambda
         write (0, '(a, 15f14.1)') ' th_s: ', dos(1)
         write (0, '(a, 15f14.5/)')' bq_j: ', p_2bq(1)
         write (0, '(a, 15f14.5/)')' r_gr: ', sbgr(1)
         write (0, '(a, 15f14.2)') ' S''/Q: ', sq(1)
c
         write (iwr, '(a/a/3a, i4)') trl, 
     &    '# Input parameters:', trl, dname, '  Stand Age  = ', age
         write (iwr, '(a, i3)') '##  ijob = ', ijob
         write (iwr, '(3(a, f5.1), a, f7.1)') 
     &      ' th_s = ', dos(1), '    th_view = ', thetvs,
     &      '    phi = ', phivs, '    Spectral band ', rlambda
         write (iwr, '(a, 15f14.5/)')' bq_j: ', p_2bq(1)
         write (iwr, '(a, 15f14.5/)')' r_gr: ', sbgr(1)
         write (iwr, '(a, 15f14.2)') ' S''/Q: ', sq(1)
c
         write (iwr, '(4a, 4(8x, a))') '# thetv', '     refl  ',
     & '     r_ground', '    b_down', 'gfr', 'rcr1', 'rgr1', 'rdif'
         write (iwr, '(f7.1, 7f12.6)') thetvs, p_2bq(1), sbgr(1),
     &         b_down(1), a_av, b_qc1(1), b_qg1(1), b_qm(1)
c
         goto 777
      endif
c
      if (ijob .eq. 1) then
         write (iwr, '(a/, 3(a, f5.1), /a, i6, a)') trl, 
     &      '#   Sun zenith = ', dos(1), '  View zenith = ', 
     &      thetvs, '  View azimuth = ', phivs,
     &      trl, nchnl, ' spectral bands'
         write (iwr, '(a, 4(6x, a), 3(9x, a))') '# wl, nm', 'refl.', 
     &      'b_down', ' r_ground','S''/Q','rcr1', 'rgr1', 'rdif'
         write (iwr, '(a)') trl
      endif
c
551   continue
      if (ijob .eq. 1) then
         write (iwr, '(f8.1, 2x, 7g13.5)')
     &      wlarr(jw), p_2bq(1), b_down(1), 
     &      sbgr(1), sqarr(nknots+1, jwl), b_qc1(1), b_qg1(1), b_qm(1)
c
         if (mod(jw, 10) .eq. 0) 
     &      write (0, '(f8.1, g16.5)') wlarr(jw), p_2bq(1)
c
         jp = 2
c
         if (jw .lt. nchnl) then 
            goto 550
         endif
c                              ********** End of the spectrum loop
         goto 777
      endif
c
      if (ijob .eq. 2) then
         write (iwr, '(a/3(a, f5.1), /a, f6.1, a, f5.2)') 
     &         trl, '#   Sun zenith = ', dos(1), 
     & '   View azimuth = ', phivs, '   View zenith step = ', dthtvs, 
     & '#   Wavelength = ', wlarr(1), ' nm     S''/Q = ', 1.d0 - skyl
         write (iwr, '(a)') trl
*     write (iwr, '(4a, 4(8x, a))') '#thetv', '  phi', '    refl' 
      write (iwr, '(4a, 4(8x, a))') '# thetv', '     refl  ', 
     & '     r_ground', '    b_down', 'gfr', 'rcr1', 'rgr1', 'rdif'
c
         ntht  = nview
c
         do jv = 1, ntht
            thetv = (thtv0 + jv*dthetv)/dr
            if (abs(thetv) .lt. dr) thetv   = 0.d0
            write (iwr, '(f7.1, 7f12.6)') thetv, rthet(jv), rsthet(jv), 
     &          bdown(jv), aavth(jv), rcr1(jv), rgr1(jv), rdif(jv)
         enddo
c                              ********** End of the AD loop
         goto 777
      endif
c
c                ******************  Several Sun zenith angles
      if (ijob .eq. 3) then
         write (0, '(2(a, f5.1), a, f7.1, i8, a)')
     &      ' thv = ', thetvs, '    phi = ', phivs, 
     &      '   Spectral band ', rlambda, nsun, ' Sun zenith angles'
         write (iwr, '(a/, 2(a, f5.1), a, f7.1, i8, a/a)') trl, 
     &      '# thv = ', thetvs, '    phi = ', phivs, 
     &      '   Spectral band ', rlambda, nsun, ' Sun zenith angles',
     &      '# ths      refl        r_gr        S''/Q'
         do js = 1, nsun
            write (iwr, '(f6.1, 3f12.5)')
     &         dos(js), p_2bq(js), sbgr(js), sq(js)
         enddo
         goto 777
      endif
c
**********************************
c
c                                         ! Output results, inversion
772   continue
c                                         ! save the vector of parameters
      open (7, file = outx, status = 'unknown')
         rewind 7
         write (7, '(d27.17)') (xx(i), i = 1, nvar) 
      close (7)
c
      print *, char(7)
         open (iwr, file = oufile, status = 'old')
         do j = 1, 6000
            read (iwr, '(1x)', end = 65)
         enddo
65       continue
         backspace iwr
         write (iwr, '(/a/a, e20.14, a, i4, a, i5, a, i3, a, f7.5)')
     &    trl, '# f = ', fff, '   iter = ', iter, 
     &    '   ncount = ', ncount, '  ier = ', ier 
c
         write (iwr, '(a/2a, 5x, i3, a, i3, 5x, a)') trl, trl, dname, 
     &    nchnl, ' band(s)'
         write (iwr, '(a, 14f7.1)') 
     &    '#  Sun angles:  ', (dos(i), i = 1, nsun)
c
         write (iwr, '(a, 47x, 25(i4, 4x))') trl,  (i, i = 1, ncl)
         write (iwr, '(a, 26x, 27a8)') trl,  'xmin  ', '  xmax  ', 
     &     '   dx   ', (liik(i), i = 1, ncl), 'totals  '
c
      write (iwr, '(a, 48x, 25a8)') trl, (kuju(i), i = 1, ncl)
      jx = 1
      if (ncl .ge. 2) then
         write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, 
     & jx, tex(jx), xmin(jx), xmax(jx), xdelt(jx), xx(jx), 
     & (xc(jcl, jx), jcl = 2, ncl), x_m(jx)
      else
         write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, 
     & jx, tex(jx), xmin(jx), xmax(jx), xdelt(jx), xx(jx), x_m(jx)
      endif
      j2 = 14
      do jx = 2, j2
         if (ncl .ge. 2) then
            write (iwr, '(a, i3, 1x, a20, 28f8.3)') trl, 
     & jx, tex(jx), xmin(jx), xmax(jx), xdelt(jx), xx(jx),
     & (xc(jcl, jx), jcl = 2, ncl), x_m(jx) 
         else
            write (iwr, '(a, i3, 1x, a20, 28f8.3)') trl,
     & jx, tex(jx), xmin(jx), xmax(jx), xdelt(jx), xx(jx), x_m(jx)
         endif
      enddo
c
      write (iwr, '(a, a26, 19x, 15i8)') trl, 
     & '# of leaf components:', (ncomp3(jcl), jcl = 1, ncl)
      do jc = 1, ncmax
         jx = j2 + jc
         if (ncl .ge. 2) then
            write (iwr, '(a, i3, 1x, a20, 28f8.3)') trl, 
     & jx, tex(jx), xmin(jx), xmax(jx), xdelt(jx), xx(jx),
     & (xc(jcl, jx), jcl = 2, ncl)
         else
            write (iwr, '(a, i3, 1x, a20, 28f8.3)') trl, 
     & jx, tex(jx), xmin(jx), xmax(jx), xdelt(jx), xx(jx)
         endif
      enddo
c
      if (lml) then
         jt = 25
         j2 = jt + 3
      else
         jt = 27
         j2 = jt + 1
      endif
      j1 = jt
      do jx = j1, j2
         if (ncl .ge. 2) then
            write (iwr, '(a, i3, 1x, a20, 28f8.3)') trl, 
     & jx, tex(jx), xmin(jx), xmax(jx), xdelt(jx), 
     & xx(jx), (xc(jcl, jx), jcl = 2, ncl)
         else
            write (iwr, '(a, i3, 1x, a20, 28f8.3)') trl, 
     & jx, tex(jx), xmin(jx), xmax(jx), xdelt(jx), xx(jx)
         endif
      enddo
c
      write (iwr, '(a, a23, 17(2x, a24))') trl, 
     & '     bark refl. files: ', (brreflf(jcl), jcl = 1, ncl)
      write (iwr, '(a, a23, 17(2x, a24))') trl, 
     & '     trunk refl. files:', (trreflf(jcl), jcl = 1, ncl)
      write (iwr, '(3(a, f6.3))') '#    rl_eff = ', r_leff,
     & '  tl_eff = ', t_leff, '  n_eff = ', r_neff
      write (iwr, '(a, 4x, a19, 25x, 26f8.3)') trl, 
     & 'leaf area density ', (u_uu(jcl), jcl = 1, ncl)
      write (iwr, '(a, 4x, a9, 2x, f8.3)') trl, 
     & 'Total LAI', t_lai
      write (iwr, '(a, 4x, a9, 2x, f8.3)') trl, 
     & 'Total BAI', x_m(12)
      write (iwr, '(a/a, f6.3, 9x, a, f6.3)') trl, 
     & '#  crown closure = ', v_liit, 'canopy closure = ', c_ano
c
      write (iwr, '(a/a)') trl, '#*** Ground vegetation, upper layer'
      jx = 28
      j2 = 8
      jt = 28
      do jr = 1, j2
         jx = jx + 1
         jt = jt + 1
         write (iwr, '(a, i3, 1x, a20, 28f8.2)') trl, 
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx), xx(jx)
      enddo
c
      write (iwr, '(a, a17, 2x, 15a8)') trl, 'Leaf model: ', lmod2
      write (iwr, '(a, a26, i4)') trl, 
     & '# of leaf components:', ncomp2
      jt = 14
      do jc = 1, ncomp2
         jx = jx + 1
         jt = jt + 1
         write (iwr, '(a, 7x, 15a18)') trl, fcoef2(jc)
         write (iwr, '(a, i3, 1x, a20, 28f8.2)') trl, 
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx), xx(jx)
      enddo
c
      if (lmod2 .eq. 'liberty') then
         jt = 24
         j1 = 47
         j2 = j1 + 2
      else
         jt = 26
         j1 = 49
         j2 = j1
      endif
      do jx = j1, j2
         jt = jt + 1
            write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl,
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx), xx(jx)
      enddo
c
      write (iwr, '(a)') '#*** Ground vegetation, lower layer'
      j1 = 50
      j2 = j1 + 6
      jt = 28
      do jx = j1, j2
         jt = jt + 1
         if (jt .eq. 32) jt = jt + 1
            write (iwr, '(a, i3, 1x, a20, 28f8.2)') trl,
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx), xx(jx)
      enddo
c
      write (iwr, '(a, a17, 2x, 15a8)') trl, 'Leaf model: ', lmod1
      write (iwr, '(a, a26, i4)') trl,
     & '# of leaf components:', ncomp1
      jt = 14
      jx = j2
      do jc = 1, ncomp1
         jx = jx + 1
         jt = jt + 1
         write (iwr, '(a, 7x, 15a18)') trl, fcoef1(jc)
         write (iwr, '(a, i3, 1x, a20, 28f8.2)') trl, 
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx), xx(jx)
      enddo
c
      if (lmod1 .eq. 'liberty') then
         j1 = 67
         jt = 24
         j2 = j1 + 2
      else
         j1 = 69
         jt = 26
         j2 = j1
      endif
      do jx = j1, j2
         jt = jt + 1
            write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, 
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx), xx(jx)
      enddo
c
      jt = 36
      j1 = 70
      j2 = j1 + 3
      do jx = j1, j2
         jt = jt + 1
            write (iwr, '(a, i3, 1x, a20, 28f8.4)') trl, 
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx), xx(jx)
      enddo

c
      if (ier .eq. 1) then 
      write (6, '(a, e20.14, a, i4, a, i5, a, i3)')
     &   '  f(x) = ', fff, '   iter = ', iter,
     &   '   ncount = ', ncount, '  ier = ', ier
      write (iwr, '(a/a, g11.4, a, i3, 4x, a, 20i3)') trl, 
     &   '# f(x) =', fff, '  n =', nxx, 'll = ', (llxx(i), i = 1, nxx)
c
         if (ijob .eq. 4 .or. ijob .eq. 5) then
            write (iwr, '(a/a, f5.1, a, f6.1/a)') trl, 
     &        '# thv = ', thetvs, '   phi = ', phivs, 
     &        '# lambda measured_refl, d_rho, fitted_refl.'
            do jths = 1, nsun
               write (iwr, '(a, f6.1)') 
     &            '# ths = ', dos(jths)
               write (iwr, '(f10.0, 3f10.4)') (wlarr(jw),
     &            bqjarr(jths, jw), drhoarr(jths, jw), 
     &            r_out(jths, jw), jw = 1, nchnl)
            enddo
         endif
c
         if (ijob .eq. 6) then
            dmean = 0.d0
            write (iwr, '(a/a, f5.1, 4x, a, f7.1/a)') trl, 
     &     '# Sun zenith angle', dos(1), 'Spectral band', wlarr(1),
     &   '#   thv   phiv      refl_in   refl_ou   delta     ratio'
            do jr = 1, nrefl
               write (iwr, '(2f8.1, 4f10.4)')
     &         thvr(jr), phir(jr), bqjarr(1, jr), r_out(1, jr),
     &         r_out(1, jr) - bqjarr(1, jr), r_out(1, jr)/bqjarr(1, jr)
               dmean = dmean + abs(r_out(1, jr) - bqjarr(1, jr))
            enddo
            dmean = dmean/nrefl
            write (iwr, '(a, f10.4)') '# delta_mean: ', dmean
         endif
c
         goto 776
      else
c                                            ! read flow.dat
         read (4, *, err = 98) jj
         jl = jl + 1
         if (jj .eq. 1) then 
            goto 400
         else if (jj .eq. 7) then
            goto 776
         else
            print *, 'Error in the file ''flow.dat'''
            goto 776
         endif
c
      endif
c
**********************************
**********************************
776   continue
      close (4)
777   continue
      close (iwr)
      print *, char(7)
c
      stop 'FRT13'
c
c     ***********************
c     ***********************
c
c                                ! Error messages
88    continue
      jwl = nint((rlambda - (400.d0 - wlstep))/wlstep)
      print *, 'MSFR: Wavelength out of range'
      write (*, '(a, i6, g15.4)')
     &         'jw, lambda:', jwl, rlambda
      write (iwr, '(a, i6, g15.4 /)')
     &         'jw, lambda:', jwl, rlambda
      write (iwr, '(/)')
      close (iwr)
      stop 'FRT13'
c
97    write (*, '(3a)') 
     &      'Error in the file ', fname, 
     &      ' - reached the end of file'
      stop 'FRT13'
98    write (*, '(3a, i4)') 
     &      'Error in the file ', fname, 'line ', jl
      stop 'FRT13'
99    write (*, '(a)')  'File not found: ', fname
      stop 'FRT13'
      end
*
************************************************************************
*
