      function func
     & (nxx, xxf)
c
c  The interface between the main program FRT and subroutines for the
c  calculation of forest reflectance and transmittance, and the merit
c  function for the inversion of the FRT.  A. Kuusk, 24.10.1995 - Nov. 2009
c  In the iversion the reflectance and/or transmittance at several
c  wavelengths and view directions can be used.
c
c     Elliptical LAD for tree foliage   12.2012
c
c     The allometric regressions for tree parameters in the 
c     subroutine regre, Feb. 1998, Dec. 2000
c
c     Fisher's grouping index, March 1998
c
c     Ground albedo is calculated integrating over polar angle 
c     in the principal plane phi = pi/2, A. Kuusk  7.10.1998
c  
c     The expert guess in the merit function, May 2001.
c
c     The 1997 version of PROSPECT, May 2001.
c
c     Two-layer homogeneous model ACRM for the ground vegetation,
c     and two leaf optics models (PROSPECT and LIBERTY), Feb. 2002.
c
c    Exchange of functions and parameters between frt.f and func using 
c    arrays ?_* in common areas
c
      include 'frtpar.h'
      include 'nch.h'
      include 'func.h'
      include 'spec.h'
      include 'spec3.h'
c
      save lelli
      save stdns, htr, hc1, hc2, rcr, dbh, rlai, rbai, clmpst,
     & clmpsh, vaicl, crncl, slwcl, rmass, eln3, thm3
      save brrarr, trrarr, wlarr, sqarr, p14, thvr, phir
      save jr, nrefl, f
c
      save glmp, uuu, efflai, tlty, ggmean, aas, aav, psgvu, 
     & bdgfu, bdgfd, btr1uk, btr1dk
c
      save ncl, shl, nzs, pkhair, ghg, 
     & thss, rsl2, rsl3, rsl4, nsun, nsangl, nchnl, at, 
     & thetv, thets, thmj, eej, ellmyj, ellsij
      save lopt, lstruc, lalbedo, thknot, phi, tlaief, tbai
      save x0, xx, xa, xb, xdelt, llxx
      save lmod3, ncomp3, ccomp1, ccomp2, ccomp3, 
     & dclcl3, aspcl3, gg3, dcell0
c
      save rsdgrou, rddgrou
      save rty, rrs, ttt, utot, rteff, tteff
c
      save lmod2, ncomp2, ul2, sl2, szz, eln2, thm2,
     & slw2, rnc2, vai2, dcell2, asp2,
     & lmod1, ncomp1, ul1, sl1, eln1, thm1,
     & slw1, rnc1, vai1, dcell1, asp1, bangs 
c
      logical lopt, lstruc, lelli, lalbedo 
      logical lmass, lbai, lhtr, lrcrwn, lhc1
      character*10 lmod1, lmod2, lmod3
c
      dimension lelli(nclmax)
      dimension stdns(nclmax), htr(nclmax), 
     & hc1(nclmax), hc2(nclmax), rcr(nclmax), dbh(nclmax),
     & rmass(nclmax), slwcl(nclmax), eln3(nclmax), thm3(nclmax),
     & shl(nclmax), rlai(nclmax), rbai(nclmax), clmpst(nclmax),
     & clmpsh(nclmax), lmod3(nclmax), ncomp3(nclmax), 
     & ccomp3(nclmax, ncomp0), vaicl(nclmax), crncl(nclmax), 
     & ccomp2(ncomp0), ccomp1(ncomp0)
      dimension dclcl3(nclmax), aspcl3(nclmax)
      dimension brrarr(nclmax, nspchnl), trrarr(nclmax, nspchnl)
c
      dimension wlarr(nspchnl)
      dimension thvr(nspchnl), phir(nspchnl)
c
*     dimension aqjarr(nangls, nspchnl), adjarr(nangls, nspchnl)
      dimension bqjarr(nangls, nspchnl), 
     & drhoarr(nangls, nspchnl)
c
      dimension thknot(nangls), p14(nknots), ggmean(nangls)
c
      dimension llxx(nvar), x0(nvar), xx(nvar), xa(nvar), 
     &          xb(nvar), xdelt(nvar), xxf(nvar)
c
      dimension glmp(nclmax)
      dimension sqarr(nangls, nspchnl)
c
      dimension uuu(nclmax)
      dimension aas(nangls), psgvu(nangls)
      dimension bdgfu(nangls, nclmax), btr1uk(nangls, nclmax)
      dimension bdgfd(nangls, nclmax), btr1dk(nangls, nclmax)
      dimension gg3(nangls, nclmax)
c
      dimension rlfcl(nclmax), rbrnc(nclmax), rtrnk(nclmax)
      dimension rnlf(nclmax), tlfcl(nclmax), ghg(nclmax)
c
      dimension rrs(nclmax, nspchnl), ttt(nclmax, nspchnl)
      dimension p2bq(nangls), bdown(nangls)
*    & , totrans(nangls), diftrans(nangls)
      dimension uuz(400)
c
      common /par/ zeps, tol, tiny, alphopt, betopt, gammopt, 
     & ier, ncount, nfmax, itmax, itbr, nbrak, ilo
c
      data lmass/.true./, lbai/.true./, lrcrwn/.true./
      data lhc1/.true./, lhtr/.true./
      data lstruc/.true./, lopt/.true./, lalbedo/.true./
c
      ncount = ncount + 1
      f   = 0.d0
c
      lopt   = l_opt
      lstruc = l_struc
      lalbedo = .true.
      ijob   = i_job 
      nsangl = n_sangl
      do jths = 1, nspchnl
         wlarr(jths) = w_larr(jths)
         do j = 2, nsangl
            sqarr(j, jths) = s_qarr(j, jths)
         enddo
      enddo
c
      if (ncount .eq. 1) then
         ncl     = n_cl
         nzs     = n_zs
         nsun    = n_sun
         nchnl   = n_chnl
         at      = a_t
         dcell0  = d_cel0
         pkhair  = p_khair
         bangs   = -0.1d0
         thss    = t_hss
         lmod2  = l_mod2
         ncomp2 = n_cmp2
         lmod1  = l_mod1
         ncomp1 = n_cmp1
c
         do jcl = 1, ncl
            do jw = 1, nspchnl
               brrarr(jcl, jw) = b_rrarr(jcl, jw)
               trrarr(jcl, jw) = t_rrarr(jcl, jw)
            enddo
            lelli(jcl)  = l_elli(jcl)
            stdns(jcl)  = x_c(jcl, 1)
            htr(jcl)    = x_c(jcl, 2)
            hc1(jcl)    = x_c(jcl, 3)
            hc2(jcl)    = x_c(jcl, 4)
            rcr(jcl)    = x_c(jcl, 5)
            dbh(jcl)    = x_c(jcl, 6)*1.d-2
            rmass(jcl)  = x_c(jcl, 7)
            slwcl(jcl)  = x_c(jcl, 8)
            eln3(jcl)   = x_c(jcl, 9)
            thm3(jcl)   = x_c(jcl, 10)*dr
            shl(jcl)    = x_c(jcl, 11)
            if (shl(jcl) .le. 0.d0) shl(jcl) = .01d0
            rlai(jcl)   = stdns(jcl)*rmass(jcl)*1.d3/slwcl(jcl)
            rbai(jcl)   = x_c(jcl, 12)*rlai(jcl)
            clmpst(jcl) = x_c(jcl, 13)
            call iterats(clmpst(jcl), glmp(jcl))
            ghg(jcl)    = x_c(jcl, 14)
            clmpsh(jcl) = c_lmpsh(jcl)
            lmod3(jcl)  = l_mod3(jcl)
            ncomp3(jcl) = n_cmp3(jcl)
            do jc = 1, ncomp3(jcl)
               ccomp3(jcl, jc) = x_c(jcl, jc+14)
            enddo
            if (lmod3(jcl) .eq. 'liberty') then
               dclcl3(jcl) = x_c(jcl, 25)
               aspcl3(jcl) = x_c(jcl, 26)
            endif
            vaicl(jcl) = x_c(jcl, 27)/clmpsh(jcl)
            crncl(jcl) = x_c(jcl, 28)
         enddo
c
         do jx = 1, nvar
            llxx(jx)  = l_lxx(jx)
            x0(jx)    = x_0(jx)
            xx(jx)    = x_x(jx)
            xa(jx)    = x_a(jx)
            xb(jx)    = x_b(jx)
            xdelt(jx) = x_delt(jx)
         enddo
c
         if (ijob .eq. 4 .or. ijob .eq. 5) then
            do jths = 1, nsangl
               do jw = 1, nspchnl
*                 aqjarr(jths, jw)  = a_qjarr(jths, jw)
*                 adjarr(jths, jw)  = a_djarr(jths, jw)
                  bqjarr(jths, jw)  = b_qjarr(jths, jw)
                  drhoarr(jths, jw) = d_rhoarr(jths, jw)
               enddo
            enddo
         endif
c
         if (ijob .eq. 6) then
            nrefl  = n_refl
            jths   = 1
            do jr = 1, nrefl
               thvr(jr) = t_hvr(jr)
               phir(jr) = p_hir(jr)
               bqjarr(jths, jr)  = b_qjarr(jths, jr)
               drhoarr(jths, jr) = d_rhoarr(jths, jr)
            enddo
         endif
c
         do ja = 1, nknots
            p14(ja) = p_14(ja)
         enddo
c
      endif
c
      phi = p_hi
      do jths = 1, nsangl
         thknot(jths) = t_hknot(jths)
      enddo
c                                ! check constraints and scale variables
      if (ijob .gt. 3) then
         do i = 1, nxx
            li  = llxx(i)
            xi3 = xxf(i)
            if (li .eq. 2) lhtr   = .false.
            if (li .eq. 3) lhc1   = .false.
            if (li .eq. 5) lrcrwn = .false.
            if (li .eq. 7) lmass  = .false.
            if (li .eq. 9) lbai   = .false.
            x2  = 0.d0
            if (xi3 .lt. -5.d0) then
               x2  = xi3 + 5.d0
               xi3 =  -5.d0
            else if (xi3 .gt. 5.d0) then
               x2  = xi3 - 5.d0
               xi3 = 5.d0
            endif
            xli    = xi3/xb(li) + xa(li)
            xx(li) = xli
            xi4    = x2*x2*at
            f      = f + xi4 + ((xli - x0(li))/xdelt(li))**2
         enddo
      endif
c                                   ! initialisation of model parameters
c
      stdns(1) = xx(1)
      htr(1)   = xx(2)
      hc1(1)   = xx(3)
c
      if (hc1(1) .gt. htr(1)) then
         hc1(1) = htr(1)
         print *, 'FUNCIFRT:'
         print *, 'Warning: tree height is less than crown length,'
         print *, 'the crown length is adjusted.'
      endif
c
      hc2(1)   = xx(4)
      rcr(1)   = xx(5)
      dbh(1)   = xx(6)*1.d-2
      rmass(1) = xx(7)
      slwcl(1) = xx(8)
      eln3(1)  = xx(9)
      thm3(1)  = xx(10)*dr
      shl(1)   = xx(11)
      if (shl(1) .le. 0.d0) shl(1) = .01d0
c
c
      if (lhtr .or. lhc1 .or. lrcrwn .or. lmass .or. lbai) then
         call regre
     & (lmass, lbai, lhtr, lrcrwn, lhc1,
     & xx, rlai, htr, hc1, rcr)
      endif
c
      rlai(1) = stdns(1)*rmass(1)*1.d3/slwcl(1)
      rbai(1) = xx(12)*rlai(1)
c
         clmpx     = xx(13)
         clmpst(1) = clmpx
         call iterats(clmpx, fgx)
         glmp(1)   = fgx
         ghg(1)    = xx(14)
c
      do jc = 1, ncomp3(1)
         ccomp3(1, jc) = xx(jc+14)
      enddo
      if (lmod3(1) .eq. 'liberty') then
         dclcl3(1) = xx(25)
         aspcl3(1) = xx(26)
      endif
      vaicl(1) = xx(27)/clmpsh(1)
      crncl(1) = xx(28)
      ul2   = xx(29)
      sl2   = xx(30)
      clmp2 = xx(31)
      szz   = xx(32)
      eln2  = xx(33)
      thm2  = xx(34)*dr
      rnc2  = xx(35)
      slw2  = xx(36)
      do jc = 1, ncomp2
         ccomp2(jc) = xx(jc+36)
      enddo
      if (lmod2 .eq. 'liberty') then
            dcell2 = xx(47)
            asp2   = xx(48)
      endif
      vai2  = xx(49)/clmp2
      ul1   = xx(50)
      sl1   = xx(51)
      clmp1 = xx(52)
      eln1  = xx(53)
      thm1  = xx(54)*dr
      rnc1  = xx(55)
      slw1  = xx(56)
      do jc = 1, ncomp1
         ccomp1(jc) = xx(jc+56)
      enddo
      if (lmod1 .eq. 'liberty') then
            dcell1 = xx(67)
            asp1   = xx(68)
      endif
      vai1  = xx(69)/clmp1
      rsl1  = xx(70)
      rsl2  = xx(71)
      rsl3  = xx(72)
      rsl4  = xx(73)
c
      do i = 1, nxx
         li = llxx(i)
         if (li .le. 10 .or. li .eq. 26) lstruc = .true.
      enddo
c
      if (ijob .lt. 6) then
c                                        ! Loop for wavelength
      do 40 jw = 1,  nchnl
c
         rlambda = wlarr(jw)
         jwl     = nint((rlambda - (400.d0 - wlstep))/wlstep)
c
      if (ijob .gt. 3) then
         do i = 1, nxx
            li = llxx(i)
            if ((li .ge. 12 .and. li .le. 25) .or. (li .eq. 8)) 
     &         lopt = .true.
         enddo
      endif
c
         if (lopt) then
            lalbedo = .true.
            rind = refr(jwl)
c                                     ! Leaf reflectance and transmittance
            do jcl = 1, ncl
               dkl = 0.d0
               do jc = 1, ncomp3(jcl)
                  dkl = dkl + ccomp3(jcl, jc)*acoef3(jcl, jwl, jc)
               enddo
               slw = slwcl(jcl)
               vai = vaicl(jcl)
               if (lmod3(jcl) .eq. 'prospect') then
                  dkl = dkl/vai*slw*1.d-6
                  call prospect(rind, dkl, vai, refl, tran)
               else if (lmod3(jcl) .eq. 'liberty') then
                  coeff = dkl*dclcl3(jcl)/dcell0*slw*1.d-6/vai
                  call liberty
     & (thknot, p14, rind, coeff, vai, aspcl3(jcl), rinf, refl, tran)
                  if (rinf .lt. 0d0) then
                     write (0, '(/a/a, f6.1/)') 'LIBERTY:', 
     &        'Cannot calculate leaf reflectance, lambda = ', rlambda
                     do jths = 1, nsun
                        p_2bq(jths)  = -99.d0
                        b_down(jths) = -99.d0
                     enddo
                     goto 40
                  endif
               endif
c
               rnlfi      = rind*crncl(jcl)
               if (rnlfi .lt. 1.d0) rnlfi = 1.d0
               rnlf(jcl)  = rnlfi
               rlfcl(jcl) = refl - ((1.d0 - rnlfi)/(1.d0 + rnlfi))**2
               tlfcl(jcl) = tran
               rbrnc(jcl) = brrarr(jcl, jwl)
               rtrnk(jcl) = trrarr(jcl, jwl)
            enddo
         endif
c
         if (lstruc) then
c                                     ! G-function in tree crowns
            thvsave = thknot(1)
            thetv = 0.d0
            thknot(1) = thetv
            do jcl = 1, ncl
               thmj  = thm3(jcl)
               eej  = 1.d0 - exp(-eln3(jcl))
           ellmyj = abs(.059d0*eln3(jcl) *(thmj - 1.02d0) + .02d0)
           ellsij = abs(.0107d0 - .0216d0*eln3(jcl)*(thmj - .265d0))
               do jths = 1, nsangl
                  thets = thknot(jths)
                  call ggg1
     &                 (thets, thmj, eej, ellmyj, ellsij, ggx)
                  gg3(jths, jcl) = ggx
*            write (6, '(f8.1, 3f10.5)') thets/dr, gg1, gg2, ggx
               enddo
c
            rltreej = rmass(jcl)*1.d3/slwcl(jcl)
            rlai(jcl) = stdns(jcl)*rltreej
            tclaij = rlai(jcl)*clmpsh(jcl) + rbai(jcl)
            ri2     = rcr(jcl)**2
            if (lelli(jcl)) then
               voi = 2.d0*pi/3.d0*ri2*hc1(jcl)
            else
               voi = pi/3.d0*ri2*(hc1(jcl) + 3.d0*hc2(jcl))
            endif
            uuu(jcl) = tclaij/voi/stdns(jcl)
c   uuu(jcl) is the foliage area density in the crown of tree(jcl)
            enddo
            thknot(1) = thvsave
c
            call strmean
     & (lelli, ncl, stdns, htr, hc1, hc2, rcr, dbh, 
     & rmass, slwcl, rlai, rbai, clmpst, clmpsh, uuu, glmp, gg3, 
     & ggmean, efflai, sntr, hmtree, vhekm, vhcilm, rmcrown, dbhmean, 
     & rmassm, slwm, vliit, cano, tlty, tlaief, tlai, tbai, uuz)
c
c                              ! calculation of probabilities
            call hetk8s
     & (lelli, ncl, shl, nzs, nsangl, thknot, phi,
     & stdns, htr, hc1, hc2, rcr, dbh, gg3, glmp, uuu, 
     & aas, aav, psgvu, bdgfu, bdgfd, btr1uk, btr1dk)
c
            a_av    = aav
            v_liit  = vliit
            c_ano   = cano
            x_m(1)  = sntr
            x_m(2)  = hmtree
            x_m(3)  = vhekm
            x_m(4)  = vhcilm
            x_m(5)  = rmcrown
            x_m(6)  = dbhmean*100.d0
            x_m(7)  = rmassm
            x_m(8)  = slwm
            t_lai   = tlai
            x_m(12) = tbai
c
            do j = 1, ncl
               u_uu(j) = uuu(j)
            enddo
c                    if ijob=0 provide the vertical profile of foliage
            if (ijob .eq. 0) then
               do j = 1, 400
                  u_uz(j) = uuz(j)
               enddo
            endif
            lstruc = .false.
         endif
c
c                              ! Mean optical parameters
         call optmean
     & (jwl, ncl, stdns, htr, hc1, hc2, dbh, rlai, rbai, clmpsh, tlty,
     & tlaief, tbai, rlfcl, tlfcl, rnlf, rbrnc, rtrnk, 
     & rleff, tleff, rneff, rty, rrs, ttt, utot, rteff, tteff)
c
         r_leff  = rleff
         t_leff  = tleff
         r_neff  = rneff
c
c                   ! Albedo of ground vegetation is calculated 
c                   ! integrating ACRM in perpendicular plane, phi = pi/2
c
         if (lalbedo) then
            phisave = phi
            phi     = pi/2.d0
            thvsave = thknot(1)
c
            call aground
     & (thknot, phi, p14,  
     & rlambda, bangs, pkhair, dcell0, 
     & lmod2, ncomp2, ul2, sl2, szz, eln2, thm2,
     & slw2, rnc2, vai2, dcell2, asp2, ccomp2, 
     & lmod1, ncomp1, ul1, sl1, eln1, thm1,
     & slw1, rnc1, vai1, dcell1, asp1, ccomp1,
     & thss, rsl1, rsl2, rsl3, rsl4, 
     & rsogrou, rdsgrou, rsdgrou, rddgrou)
c                             ! rddgrou = albedo of ground vegetation
c
            phi     = phisave
            thknot(1) = thvsave
         endif
c
c                                   ! Direct run of the model
         call het48o
     & (ncl, nsangl, thknot, phi, stdns, sqarr,
     & uuu, eln3, thm3, ghg, efflai, rteff, tteff, tlty, 
     & ggmean, rtrnk, aas, aav, psgvu, bdgfu, bdgfd, btr1uk, btr1dk, 
     & rlambda, bangs, pkhair, dcell0,
     & lmod2, ncomp2, ul2, sl2, szz, eln2, thm2,
     & slw2, rnc2, vai2, dcell2, asp2, ccomp2,
     & lmod1, ncomp1, ul1, sl1, eln1, thm1,
     & slw1, rnc1, vai1, dcell1, asp1, ccomp1,
     & thss, rsl1, rsl2, rsl3, rsl4, rnlf, 
     & rrs, ttt, rdsgrou, rsdgrou, rddgrou, 
     & p2bq, bdown, bqc1, bqg1, bqm)
*    & , totrans, diftrans)
c
         if (ijob .lt. 3) then
            b_qc1(jw) = bqc1
            b_qg1(jw) = bqg1
            b_qm(jw)  = bqm
         endif
c
c                                   ! calculation of the merit function
         do jths = 1, nsun
            if (ijob .gt. 3) then
*              aqi  = aqjarr(jths, jw)
*              xi1  = aqjarr(jths, jw) - totrans(jths)
*              adi  = adjarr(jths, jw)
*              xi2  = adi - diftrans(jths)
               xi4  = bqjarr(jths, jw) - p2bq(jths)
*        print *, 'HET48o: bqi, p2bq: ', bqi, p2bq(jths)
               if (ijob .eq. 5) then
                  f    = f + xi4**2
*                 if ((aqi .gt. 0.d0) .and. (adi .gt. 0.d0)) then 
*                    f    = f + xi1*xi1 + xi2*xi2
*                    f    = f + xi1*xi1
*                    f    = f + xi2*xi2
*                 endif
               else
                  f    = f + (xi4/drhoarr(jths, jw))**2
*                 f    = f + xi4*xi4/bqi
*                 if ((aqi .gt. 0.d0) .and. (adi .gt. 0.d0)) then 
*                    f    = f + xi1*xi1/aqi + xi2*xi2/adi
*                    f    = f + xi1*xi1/aqi 
*                    f    = f + xi2*xi2/adi
*                 endif
               endif
            endif
            p_2bq(jths)     = p2bq(jths)
            r_out(jths, jw) = p2bq(jths)
            b_down(jths)    = bdown(jths)
*           t_otrans(jths)  = totrans(jths)
*           d_iftrans(jths) = diftrans(jths)
         enddo
40    continue
c
      func  = f
c
      else
c                     ijob = 6
         rlambda = wlarr(1)
         jwl     = nint((rlambda - (400.d0 - wlstep))/wlstep)
         rind = refr(jwl)
c                                     ! Leaf reflectance and transmittance
         do jcl = 1, ncl
            dkl = 0.d0
            do jc = 1, ncomp3(jcl)
               dkl = dkl + ccomp3(jcl, jc)*acoef3(jcl, jwl, jc)
            enddo
            slw = slwcl(jcl)
            vai = vaicl(jcl)
            if (lmod3(jcl) .eq. 'prospect') then
               dkl = dkl/vai*slw*1.d-6
               call prospect(rind, dkl, vai, refl, tran)
            else if (lmod3(jcl) .eq. 'liberty') then
               coeff = dkl*dclcl3(jcl)/dcell0*slw*1.d-6/vai
               call liberty
     & (thknot, p14, rind, coeff, vai, aspcl3(jcl), rinf, refl, tran)
               if (rinf .lt. 0d0) then
                  write (0, '(/a/a, f6.1/)') 'LIBERTY:',
     &        'Cannot calculate leaf reflectance, lambda = ', rlambda
                  do jths = 1, nsun
                     p_2bq(jths)  = -99.d0
                     b_down(jths) = -99.d0
                  enddo
                  goto 41
               endif
            endif
c
            rnlfi      = rind*crncl(jcl)
            if (rnlfi .lt. 1.d0) rnlfi = 1.d0
            rnlf(jcl)  = rnlfi
            rlfcl(jcl) = refl - ((1.d0 - rnlfi)/(1.d0 + rnlfi))**2
            tlfcl(jcl) = tran
            rbrnc(jcl) = brrarr(jcl, jwl)
            rtrnk(jcl) = trrarr(jcl, jwl)
         enddo
c
c                                     ! G-function in tree crowns
         thetv = 0.d0
         thknot(1) = thetv
         do jcl = 1, ncl
            thmj  = thm3(jcl)
            eej  = 1.d0 - exp(-eln3(jcl))
           ellmyj = abs(.059d0*eln3(jcl) *(thmj - 1.02d0) + .02d0)
           ellsij = abs(.0107d0 - .0216d0*eln3(jcl)*(thmj - .265d0))
            do jths = 1, nsangl
               thets = thknot(jths)
               call ggg1
     &              (thets, thmj, eej, ellmyj, ellsij, ggx)
               gg3(jths, jcl) = ggx
*            write (6, '(f8.1, 3f10.5)') thets/dr, gg1, gg2, ggx
            enddo
c
            rltreej = rmass(jcl)*1.d3/slwcl(jcl)
            rlai(jcl) = stdns(jcl)*rltreej
            tclaij = rlai(jcl)*clmpsh(jcl) + rbai(jcl)
            ri2     = rcr(jcl)**2
            if (lelli(jcl)) then
               voi = 2.d0*pi/3.d0*ri2*hc1(jcl)
            else
               voi = pi/3.d0*ri2*(hc1(jcl) + 3.d0*hc2(jcl))
            endif
            uuu(jcl) = tclaij/voi/stdns(jcl)
c   uuu(jcl) is the foliage area density in the crown of tree(jcl)
         enddo
c
            call strmean
     & (lelli, ncl, stdns, htr, hc1, hc2, rcr, dbh,
     & rmass, slwcl, rlai, rbai, clmpst, clmpsh, uuu, glmp, gg3,
     & ggmean, efflai, sntr, hmtree, vhekm, vhcilm, rmcrown, dbhmean,
     & rmassm, slwm, vliit, cano, tlty, tlaief, tlai, tbai, uuz)
c
            v_liit  = vliit
            c_ano   = cano
            x_m(1)  = sntr
            x_m(2)  = hmtree
            x_m(3)  = vhekm
            x_m(4)  = vhcilm
            x_m(5)  = rmcrown
            x_m(6)  = dbhmean*100.d0
            x_m(7)  = rmassm
            x_m(8)  = slwm
            t_lai   = tlai
            x_m(12) = tbai
c
            do j = 1, ncl
               u_uu(j) = uuu(j)
            enddo
c                              ! Mean optical parameters
         call optmean
     & (jwl, ncl, stdns, htr, hc1, hc2, dbh, rlai, rbai, clmpsh, tlty,
     & tlaief, tbai, rlfcl, tlfcl, rnlf, rbrnc, rtrnk,
     & rleff, tleff, rneff, rty, rrs, ttt, utot, rteff, tteff)
c
         r_leff  = rleff
         t_leff  = tleff
         r_neff  = rneff
c                   ! Albedo of ground vegetation is calculated
c                   ! integrating ACRM in perpendicular plane, phi = pi/2
            phi     = pi/2.d0
c
            call aground
     & (thknot, phi, p14,
     & rlambda, bangs, pkhair, dcell0,
     & lmod2, ncomp2, ul2, sl2, szz, eln2, thm2,
     & slw2, rnc2, vai2, dcell2, asp2, ccomp2,
     & lmod1, ncomp1, ul1, sl1, eln1, thm1,
     & slw1, rnc1, vai1, dcell1, asp1, ccomp1,
     & thss, rsl1, rsl2, rsl3, rsl4,
     & rsogrou, rdsgrou, rsdgrou, rddgrou)
c                             ! rddgrou = albedo of ground vegetation
c
      do jr = 1, nrefl
         thknot(1) = thvr(jr)
         phi       = phir(jr)
c     
c                              ! calculation of probabilities
            call hetk8s
     & (lelli, ncl, shl, nzs, nsangl, thknot, phi,
     & stdns, htr, hc1, hc2, rcr, dbh, gg3, glmp, uuu, 
     & aas, aav, psgvu, bdgfu, bdgfd, btr1uk, btr1dk)
c
c                                   ! Direct run of the model
         call het48o
     & (ncl, nsangl, thknot, phi, stdns, sqarr,
     & uuu, eln3, thm3, ghg, efflai, rteff, tteff, tlty,
     & ggmean, rtrnk, aas, aav, psgvu, bdgfu, bdgfd, btr1uk, btr1dk,
     & rlambda, bangs, pkhair, dcell0,
     & lmod2, ncomp2, ul2, sl2, szz, eln2, thm2,
     & slw2, rnc2, vai2, dcell2, asp2, ccomp2,
     & lmod1, ncomp1, ul1, sl1, eln1, thm1,
     & slw1, rnc1, vai1, dcell1, asp1, ccomp1,
     & thss, rsl1, rsl2, rsl3, rsl4, rnlf,
     & rrs, ttt, rdsgrou, rsdgrou, rddgrou,
     & p2bq, bdown, bqc1, bqg1, bqm)
c
c                                   ! calculation of the merit function
         jths = 1
         xi4  = bqjarr(jths, jr) - p2bq(jths)
         f    = f + (xi4/drhoarr(jths, jr))**2
         r_out(jths, jr) = p2bq(jths)
      enddo
      func = f
c
41    endif
*           print *, 'f  = ', f
      return
      end
*
************************************************************************
*
