      subroutine hetk8o
     & (ncl, jwl, nsangl, thknot, phi, stdns, uuu, eln3, thm3, 
     & ghg, bdgfu, bdgfd, btr1uk, btr1dk, pkhair, rnlf, rtrnk, 
     & rrs, ttt, bi0u, bi0d)
c
c     Replaced the phase function by M. Mõttus by the Henyey-Greenstein
c     phase function, 01.2013
c
c     Phase function of conifer shoots by M. Mõttus (Agricult. & Forest
c     Meteorol., 154-155, 67-74 (2012)), 01.2013
c
c     Single scattering radiance of tree crowns, up and down
c     A. Kuusk  27.11.2000
c
c  Input:
c      & (ncl, jwl, nsangl, thknot, phi, stdns, uuu,
c      & bdgfu, bdgfd, btr1uk, btr1dk, pkhair, rnlf, rtrnk, rrs, ttt,
c  Output:
c      bi0u, bi0d
c
      include 'frtpar.h'
      include 'nch.h'
c      
      dimension thknot(nangls)
      dimension stdns(nclmax), uuu(nclmax)
      dimension bdgfu(nangls, nclmax), btr1uk(nangls, nclmax)
      dimension bdgfd(nangls, nclmax), btr1dk(nangls, nclmax)
      dimension rnlf(nclmax), rtrnk(nclmax)
      dimension eln3(nclmax), thm3(nclmax), ghg(nclmax)
      dimension rrs(nclmax, nspchnl), ttt(nclmax, nspchnl)
      dimension bi0u(nangls), bi0d(nangls)
c
*     data pi/3.14159265358979d0/
      data eps/.01d0/, eps2/.0001d0/
c
      thetv   = thknot(1)
      sthetv  = sin(thetv)
      cthetv  = cos(thetv)
      cphi    = cos(phi)
      sphi    = sin(phi)
c
      if (nsangl .eq. nknots) then
         nang1 = 2
      else
         nang1 = nknots + 1
      endif
c                                    **** incident angle loop ****
      do 300 jths = nang1, nsangl
         thets   = thknot(jths)
         cthets  = cos(thets)
         sthets  = sin(thets)
         tgths   = sthets/cthets
         calph   = sthetv*sthets*cphi + cthetv*cthets
         alpha   = acos(calph)
         aalpha  = abs(alpha)
         if (aalpha .lt. eps) alpha = 0.d0
         if (alpha .lt. 0.d0) alpha = alpha + pi
         salph   = sin(alpha)
         calp2u  = cos(alpha*.5d0)
         calp2d  = sqrt((1.d0 - calph)/2.d0)
c
         bbbu    = 0.d0
         bbbd    = 0.d0
c                          ! Summing over tree classes
         do 180 jcl = 1, ncl
            ulj    = uuu(jcl)
            rdlf   = rrs(jcl, jwl)
            tleaf  = ttt(jcl, jwl)
            rnj    = rnlf(jcl)
            thmj   = thm3(jcl)
            eej    = 1.d0 - exp(-eln3(jcl))
*           if (ghg(jcl) .le. .0d0) then
              szg    = 1.d0
              ellmyj = abs(.059d0*eln3(jcl) *(thmj - 1.02d0) + .02d0)
              ellsij = abs(.0107d0 - .0216d0*eln3(jcl)*(thmj - .265d0))

c
c                     ! Phase function of scattering gamma
c                     ! the upper hemisphere (diffuse + specular)
              call gamma
     & (thetv, thets, phi, thmj, eej, szg, rdlf, tleaf, rnj,
     & pkhair, ellmyj, ellsij, gg1, gg2, gamm)
c
*                     ! Phase function gamma,  spherical orientation
*                     ! the upper hemisphere
*           gammr = (salph + (pi - alpha)*calph)/(3.d0*pi)
*           gammt = (salph - alpha*calph)/(3.d0*pi)
*           gammd = rdlf*gammr + tleaf*gammt
*           call gmfres(calp2u, rnj, pkhair, rfr)
*           gammout = gammd + rfr*.125d0
c
*           else
c
c  Phase function of specular reflection
              call gmfres(calp2u, rnj, pkhair, gmf)
              if (eej .le. eps) then
                 gl  = 1.d0
              else
                 y4  = abs(cthetv + cthets)*.5d0/calp2u
                 thp = acos(y4)
                 call gleaf(thmj, eej, thp, gl)
              endif
              gammf = gmf*gl*.125d0
              omg   = rdlf + tleaf
*	      c
*	      c  Phase function of conifer shoots, Matti Mõttus, 2012.
*              fiso  = 0.77d0*omg
*              flamb = 1.d0 - fiso
*              if (calph .le. 0.d0) then
*                 gamm = fiso
*              else
*                 gamm = fiso + 4.d0*flamb*abs(calph)
*              endif
*              gammd  = gamm/4.d0/pi*omg
*              gamm   = gammd + gammf
c
            if (ghg(jcl) .gt. .0d0) then
c  The Henyey-Greenstein phase function
              ghgj = ghg(jcl)
              gammd = (1.d0 - ghgj*ghgj)/sqrt((1.d0 + ghgj*ghgj -
     &                2.d0*ghgj*calph)**3)/(4.d0*pi)*omg
              gamm  = gammd + gammf
c
            endif
*
*       print *, 'thv, alpha, gamme, gammil, gamm, gl, gammf', 
*    &    thetv/dr, alpha/dr, gamme, gammil, gamm, gl, gammf
c
            bc1u    = bdgfu(jths, jcl)*gamm*ulj*stdns(jcl)/
     &                (cthets*cthetv)
c
c                        ! Phase function gamma, vertical orientation
c                        ! tty = 0, rfr = 0
c                        ! the upper hemisphere
            gammout = rtrnk(jcl)*(sphi - (phi - pi)*cphi)/(2.d0*pi)
c
            btr1u   = btr1uk(jths, jcl)*gammout*stdns(jcl)*
     &                tgths*sthetv/cthetv
c
            bbbu    = bbbu + bc1u + btr1u
c
c                                      ! the lower hemisphere
            gammr = (salph + (pi - alpha)*calph)/(3.d0*pi)
            gammt = (salph - alpha*calph)/(3.d0*pi)
            gammd = rdlf*gammt + tleaf*gammr
c
            if (calp2d .lt. eps2) then
              rfr = 0.d0
            else
              call gmfres(calp2d, rnj, pkhair, rfr)
            endif
c
            gammout = gammd + rfr*.125d0
c
            bc1d    = bdgfd(jths, jcl)*gammout*ulj*stdns(jcl)/
     &                (cthets*cthetv)
c                        ! Phase function gamma, vertical orientation
c                        ! tty = 0, rfr = 0
c                        ! the lower hemisphere
            gammout = rtrnk(jcl)*(sphi - phi*cphi)/(2.d0*pi)
c
            btr1d   = btr1dk(jths, jcl)*gammout*stdns(jcl)*
     &                tgths*sthetv/cthetv
            bbbd    = bbbd + bc1d + btr1d

180      continue
c 
         bi0u(jths)   = bbbu
         bi0d(jths)   = bbbd
c
300   continue
c
      return
      end
*
***********************************************************************
*
